"""
   !--------------------------------------------------------------------------!
   ! LICENSE INFO:                                                            !
   !--------------------------------------------------------------------------!
   !    This file is part of localCIDER.                                      !
   !                                                                          !
   !    Version 0.1.9                                                         !
   !                                                                          !
   !    Copyright (C) 2014 - 2016                                             !
   !    The localCIDER development team (current and former contributors)     !
   !    Alex Holehouse, James Ahad, Rahul K. Das.                             !
   !                                                                          !
   !    localCIDER was developed in the lab of Rohit Pappu at Washington      !
   !    University in St. Louis. Please see the website for citation          !
   !    information:                                                          !
   !                                                                          !
   !    http://pappulab.github.io/localCIDER/                                 !
   !                                                                          !
   !    For more information please see the Pappu lab website:                !
   !                                                                          !
   !    http://pappulab.wustl.edu/                                            !
   !                                                                          !
   !    localCIDER is free software: you can redistribute it and/or modify    !
   !    it under the terms of the GNU General Public License as published by  !
   !    the Free Software Foundation, either version 3 of the License, or     !
   !    (at your option) any later version.                                   !
   !                                                                          !
   !    localCIDER is distributed in the hope that it will be useful,         !
   !    but WITHOUT ANY WARRANTY; without even the implied warranty of        !
   !    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         !
   !    GNU General Public License for more details.                          !
   !                                                                          !
   !    You should have received a copy of the GNU General Public License     !
   !    along with localCIDER.  If not, see <http://www.gnu.org/licenses/>.   !
   !--------------------------------------------------------------------------!
   ! AUTHORSHIP INFO:                                                         !
   !--------------------------------------------------------------------------!
   !                                                                          !
   ! MAIN AUTHOR:   Alex Holehouse                                            !
   !                                                                          !
   !--------------------------------------------------------------------------!


   File Description:
   ================

   This is the main package import file for the localCIDER package.

"""

__all__ = ['sequenceParameters', 'plots']

from backend.config import VERSION as localCIDER_version
from backend.backendtools import status_message
import sequenceParameters as sequenceParameters
import sequencePermutants as sequencePermutants
import plots as plots

# Explicit exception importing
from backend.localciderExceptions import KeyFileException
from backend.localciderExceptions import PlottingException
from backend.localciderExceptions import SequenceException
from backend.localciderExceptions import SequenceFileParserException
from backend.localciderExceptions import ResTableException
from backend.localciderExceptions import WLException
status_message("localCIDER version " + localCIDER_version)



"""
   !--------------------------------------------------------------------------!
   ! LICENSE INFO:                                                            !
   !--------------------------------------------------------------------------!
   !    This file is part of localCIDER.                                      !
   !                                                                          !
   !    Version 0.1.9                                                         !
   !                                                                          !
   !    Copyright (C) 2014 - 2016                                             !
   !    The localCIDER development team (current and former contributors)     !
   !    Alex Holehouse, James Ahad, Rahul K. Das.                             !
   !                                                                          !
   !    localCIDER was developed in the lab of Rohit Pappu at Washington      !
   !    University in St. Louis. Please see the website for citation          !
   !    information:                                                          !
   !                                                                          !
   !    http://pappulab.github.io/localCIDER/                                 !
   !                                                                          !
   !    For more information please see the Pappu lab website:                !
   !                                                                          !
   !    http://pappulab.wustl.edu/                                            !
   !                                                                          !
   !    localCIDER is free software: you can redistribute it and/or modify    !
   !    it under the terms of the GNU General Public License as published by  !
   !    the Free Software Foundation, either version 3 of the License, or     !
   !    (at your option) any later version.                                   !
   !                                                                          !
   !    localCIDER is distributed in the hope that it will be useful,         !
   !    but WITHOUT ANY WARRANTY; without even the implied warranty of        !
   !    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         !
   !    GNU General Public License for more details.                          !
   !                                                                          !
   !    You should have received a copy of the GNU General Public License     !
   !    along with localCIDER.  If not, see <http://www.gnu.org/licenses/>.   !
   !--------------------------------------------------------------------------!
   ! AUTHORSHIP INFO:                                                         !
   !--------------------------------------------------------------------------!
   !                                                                          !
   ! MAIN AUTHOR:   Alex Holehouse                                            !
   !                                                                          !
   !--------------------------------------------------------------------------!


   Unit tests for functions in plots


"""

import os.path
import time
import unittest
import random
from localcider import plots, sequenceParameters
import testTools


class TestPlotsFunctions(unittest.TestCase):

    def setUp(self):
        self.rseq = sequenceParameters.SequenceParameters(
            testTools.generate_random_sequence(minLen=20, maxLen=500))

    def test_save_single_phasePlot(self):
        fp = self.rseq.get_fraction_positive()
        fn = self.rseq.get_fraction_negative()

        plots.save_single_phasePlot(fp, fn, 'tmpfiles/single_PP.png', 'TEST TITLE')

        plots.save_single_phasePlot(
            fp, fn, 'tmpfiles/single_PP_NO_LEGEND.png', 'TEST TITLE', False)

        plots.save_single_phasePlot(
            fp, fn, 'tmpfiles/single_PP_NO_LEGEND.pdf', 'TEST TITLE', False, saveFormat='pdf')

    def test_save_multiple_phasePlot(self):

        rseq2 = sequenceParameters.SequenceParameters(
            testTools.generate_random_sequence(minLen=20, maxLen=500))

        fp1 = self.rseq.get_fraction_positive()
        fn1 = self.rseq.get_fraction_negative()
        fp2 = rseq2.get_fraction_positive()
        fn2 = rseq2.get_fraction_negative()

        plots.save_multiple_phasePlot(
            [fp1, fp2], [fn1, fn2], 'tmpfiles/mult_PP', ['a', 'b'], 'TEST TITLE')
        plots.save_multiple_phasePlot([fp1,
                                       fp2],
                                      [fn1,
                                       fn2],
                                      'tmpfiles/mult_PP_NO_LEGEND.png',
                                      ['a',
                                       'b'],
                                      title='TEST TITLE',
                                      legendOn=False)

        plots.save_multiple_phasePlot([fp1,
                                       fp2],
                                      [fn1,
                                       fn2],
                                      'tmpfiles/mult_PP_NO_LEGEND_xLim1.png',
                                      ['a',
                                       'b'],
                                      title='TEST TITLE',
                                      legendOn=False,
                                      xLim=1)

        plots.save_multiple_phasePlot([fp1,
                                       fp2],
                                      [fn1,
                                       fn2],
                                      'tmpfiles/mult_PP_NO_LEGEND_xlim2.png',
                                      ['a',
                                       'b'],
                                      title='TEST TITLE',
                                      legendOn=False,
                                      xLim=2)

        plots.save_multiple_phasePlot([fp1,
                                       fp2],
                                      [fn1,
                                       fn2],
                                      'tmpfiles/mult_PP_NO_LEGEND_xlim2_ylim2.png',
                                      ['a',
                                       'b'],
                                      title='TEST TITLE',
                                      legendOn=False,
                                      xLim=2,
                                      yLim=2)

        plots.save_multiple_phasePlot([fp1,
                                       fp2],
                                      [fn1,
                                       fn2],
                                      'tmpfiles/mult_PP_NO_LEGEND_xlim2_ylim2_fs_20.png',
                                      ['a',
                                       'b'],
                                      title='TEST TITLE',
                                      legendOn=False,
                                      xLim=2,
                                      yLim=2,
                                      fontSize=20)

        plots.save_multiple_phasePlot([fp1,
                                       fp2],
                                      [fn1,
                                       fn2],
                                      'tmpfiles/mult_PP_NO_LEGEND_xlim2_ylim2_fs_20.pdf',
                                      ['a',
                                       'b'],
                                      title='TEST TITLE',
                                      legendOn=False,
                                      xLim=2,
                                      yLim=2,
                                      fontSize=20,
                                      saveFormat='pdf')

    def test_save_multiple_phasePlot2(self):

        rseq2 = sequenceParameters.SequenceParameters(
            testTools.generate_random_sequence(minLen=20, maxLen=500))

        plots.save_multiple_phasePlot2(
            [self.rseq, rseq2], 'tmpfiles/mult_PP', ['a', 'b'], 'TEST TITLE')
        plots.save_multiple_phasePlot2(
            [self.rseq, rseq2], 'tmpfiles/mult_PP_NO_LEGEND', ['a', 'b'], 'TEST TITLE', False)
        plots.save_multiple_phasePlot2([self.rseq,
                                        rseq2],
                                       'tmpfiles/mult_PP_NO_LEGEND_xLim1.png',
                                       ['a',
                                        'b'],
                                       title='TEST TITLE',
                                       legendOn=False,
                                       xLim=1)

        plots.save_multiple_phasePlot2([self.rseq,
                                        rseq2],
                                       'tmpfiles/mult_PP_NO_LEGEND_xlim2.png',
                                       ['a',
                                        'b'],
                                       title='TEST TITLE',
                                       legendOn=False,
                                       xLim=2)

        plots.save_multiple_phasePlot2([self.rseq,
                                        rseq2],
                                       'tmpfiles/mult_PP_NO_LEGEND_xlim2_ylim2.png',
                                       ['a',
                                        'b'],
                                       title='TEST TITLE',
                                       legendOn=False,
                                       xLim=2,
                                       yLim=2)

        plots.save_multiple_phasePlot2([self.rseq,
                                        rseq2],
                                       'tmpfiles/mult_PP_NO_LEGEND_xlim2_ylim2_fs_20.png',
                                       ['a',
                                        'b'],
                                       title='TEST TITLE',
                                       legendOn=False,
                                       xLim=2,
                                       yLim=2,
                                       fontSize=20)

        plots.save_multiple_phasePlot2([self.rseq,
                                        rseq2],
                                       'tmpfiles/mult_PP_NO_LEGEND_xlim2_ylim2_fs_20.pdf',
                                       ['a',
                                        'b'],
                                       title='TEST TITLE',
                                       legendOn=False,
                                       xLim=2,
                                       yLim=2,
                                       fontSize=20,
                                       saveFormat='pdf')

    def test_save_single_uverskyPlot(self):
        hydro = self.rseq.get_uversky_hydropathy()
        mnc = self.rseq.get_mean_net_charge()

        plots.save_single_uverskyPlot(
            hydro, mnc, 'tmpfiles/single_UV', 'TEST TITLE')

        plots.save_single_uverskyPlot(
            hydro,
            mnc,
            'tmpfiles/single_UV_NO_LEGEND.png',
            'TEST TITLE',
            False)

        plots.save_single_uverskyPlot(
            hydro,
            mnc,
            'tmpfiles/single_UV_NO_LEGEND.pdf',
            'TEST TITLE',
            False,
            saveFormat='pdf')

    def test_save_multiple_uverskyPlot(self):

        rseq2 = sequenceParameters.SequenceParameters(
            testTools.generate_random_sequence(minLen=20, maxLen=500))

        hy1 = self.rseq.get_uversky_hydropathy()
        mnc1 = self.rseq.get_mean_net_charge()
        hy2 = rseq2.get_uversky_hydropathy()
        mnc2 = rseq2.get_mean_net_charge()

        plots.save_multiple_uverskyPlot(
            [hy1, hy2], [mnc1, mnc2], 'tmpfiles/mult_UV', ['a', 'b'], 'TEST TITLE')
        plots.save_multiple_uverskyPlot([hy1,
                                         hy2],
                                        [mnc1,
                                         mnc2],
                                        'tmpfiles/mult_UV_NO_LEGEND.png',
                                        ['a',
                                         'b'],
                                        title='TEST TITLE',
                                        legendOn=False)

        plots.save_multiple_uverskyPlot([hy1,
                                         hy2],
                                        [mnc1,
                                         mnc2],
                                        'tmpfiles/mult_UV_NO_LEGEND_xLim1.png',
                                        ['a',
                                         'b'],
                                        title='TEST TITLE',
                                        legendOn=False,
                                        xLim=1)

        plots.save_multiple_uverskyPlot([hy1,
                                         hy2],
                                        [mnc1,
                                         mnc2],
                                        'tmpfiles/mult_UV_NO_LEGEND_xlim2.png',
                                        ['a',
                                         'b'],
                                        title='TEST TITLE',
                                        legendOn=False,
                                        xLim=2)

        plots.save_multiple_uverskyPlot([hy1,
                                         hy2],
                                        [mnc1,
                                         mnc2],
                                        'tmpfiles/mult_UV_NO_LEGEND_xlim2_ylim2',
                                        ['a',
                                         'b'],
                                        title='TEST TITLE',
                                        legendOn=False,
                                        xLim=2,
                                        yLim=2)

        plots.save_multiple_uverskyPlot([hy1,
                                         hy2],
                                        [mnc1,
                                         mnc2],
                                        'tmpfiles/mult_UV_NO_LEGEND_xlim2_ylim2_fs_20.png',
                                        ['a',
                                         'b'],
                                        title='TEST TITLE',
                                        legendOn=False,
                                        xLim=2,
                                        yLim=2,
                                        fontSize=20)

        plots.save_multiple_uverskyPlot([hy1,
                                         hy2],
                                        [mnc1,
                                         mnc2],
                                        'tmpfiles/mult_UV_NO_LEGEND_xlim2_ylim2_fs_20.pdf',
                                        ['a',
                                         'b'],
                                        title='TEST TITLE',
                                        legendOn=False,
                                        xLim=2,
                                        yLim=2,
                                        fontSize=20,
                                        saveFormat='pdf')

    def test_save_multiple_uverskyPlot2(self):

        rseq2 = sequenceParameters.SequenceParameters(
            testTools.generate_random_sequence(minLen=20, maxLen=500))

        hy1 = self.rseq.get_uversky_hydropathy()
        mnc1 = self.rseq.get_mean_net_charge()
        hy2 = rseq2.get_uversky_hydropathy()
        mnc2 = rseq2.get_mean_net_charge()

        plots.save_multiple_uverskyPlot2(
            [self.rseq, rseq2], 'tmpfiles/mult_UV.png', ['a', 'b'], 'TEST TITLE')
        
        plots.save_multiple_uverskyPlot2([self.rseq,
                                          rseq2],
                                         'tmpfiles/mult_UV2_NO_LEGEND.png',
                                         ['a',
                                          'b'],
                                         title='TEST TITLE',
                                         legendOn=False)
        
        plots.save_multiple_uverskyPlot2([self.rseq,
                                          rseq2],
                                         'tmpfiles/mult_UV2_NO_LEGEND_xLim1.png',
                                         ['a',
                                          'b'],
                                         title='TEST TITLE',
                                         legendOn=False,
                                         xLim=1)

        plots.save_multiple_uverskyPlot2([self.rseq,
                                          rseq2],
                                         'tmpfiles/mult_UV2_NO_LEGEND_xlim2.png',
                                         ['a',
                                          'b'],
                                         title='TEST TITLE',
                                         legendOn=False,
                                         xLim=2)

        plots.save_multiple_uverskyPlot2([self.rseq,
                                          rseq2],
                                         'tmpfiles/mult_UV2_NO_LEGEND_xlim2_ylim2.png',
                                         ['a',
                                          'b'],
                                         title='TEST TITLE',
                                         legendOn=False,
                                         xLim=2,
                                         yLim=2)

        plots.save_multiple_uverskyPlot2([self.rseq,
                                          rseq2],
                                         'tmpfiles/mult_UV2_NO_LEGEND_xlim2_ylim2_fs_20.png',
                                         ['a',
                                          'b'],
                                         title='TEST TITLE',
                                         legendOn=False,
                                         xLim=2,
                                         yLim=2,
                                         fontSize=20)

        plots.save_multiple_uverskyPlot2([self.rseq,
                                          rseq2],
                                         'tmpfiles/mult_UV2_NO_LEGEND_xlim2_ylim2_fs_20.pdf',
                                         ['a',
                                          'b'],
                                         title='TEST TITLE',
                                         legendOn=False,
                                         xLim=2,
                                         yLim=2,
                                         fontSize=20,
                                         saveFormat='pdf')



""" 
   !--------------------------------------------------------------------------!
   ! LICENSE INFO:                                                            !
   !--------------------------------------------------------------------------!
   !    This file is part of localCIDER.                                      !
   !                                                                          !
   !    Version 0.1.9                                                         !
   !                                                                          !
   !    Copyright (C) 2014 - 2016                                             !
   !    The localCIDER development team (current and former contributors)     !
   !    Alex Holehouse, James Ahad, Rahul K. Das.                             !
   !                                                                          !
   !    localCIDER was developed in the lab of Rohit Pappu at Washington      !
   !    University in St. Louis. Please see the website for citation          !
   !    information:                                                          !
   !                                                                          !
   !    http://pappulab.github.io/localCIDER/                                 !
   !                                                                          !
   !    For more information please see the Pappu lab website:                !
   !                                                                          !
   !    http://pappulab.wustl.edu/                                            !
   !                                                                          !
   !    localCIDER is free software: you can redistribute it and/or modify    !
   !    it under the terms of the GNU General Public License as published by  !
   !    the Free Software Foundation, either version 3 of the License, or     !
   !    (at your option) any later version.                                   !
   !                                                                          !
   !    localCIDER is distributed in the hope that it will be useful,         !
   !    but WITHOUT ANY WARRANTY; without even the implied warranty of        !
   !    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         !
   !    GNU General Public License for more details.                          !
   !                                                                          !
   !    You should have received a copy of the GNU General Public License     !
   !    along with localCIDER.  If not, see <http://www.gnu.org/licenses/>.   !
   !--------------------------------------------------------------------------!
   ! AUTHORSHIP INFO:                                                         !
   !--------------------------------------------------------------------------!
   !                                                                          !
   ! MAIN AUTHOR:    Alex Holehouse                                           !
   !                                                                          !
   !--------------------------------------------------------------------------!

   Amino acids data

"""


# <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
# <>          
# <>          THREE-TO-ONE LETTER CODE TRANSLATION
# <> 
# <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>


THREE_TO_ONE = {'ALA':'A', 
                'CYS':'C',
                'ASP':'D',
                'GLU':'E',
                'PHE':'F',
                'GLY':'G',
                'HIS':'H', 
                'ILE':'I',
                'LYS':'K',
                'LEU':'L',
                'MET':'M',
                'ASN':'N',
                'PRO':'P',
                'GLN':'Q',
                'ARG':'R',
                'SER':'S',
                'THR':'T',
                'VAL':'V',
                'TRP':'W',
                'TYR':'Y'}

ONE_TO_THREE = {'A':'ALA', 
                'C':'CYS',
                'D':'ASP',
                'E':'GLU',
                'F':'PHE',
                'G':'GLY',
                'H':'HIS', 
                'I':'ILE',
                'K':'LYS',
                'L':'LEU',
                'M':'MET',
                'N':'ASN',
                'P':'PRO',
                'Q':'GLN',
                'R':'ARG',
                'S':'SER',
                'T':'THR',
                'V':'VAL',
                'W':'TRP',
                'Y':'TYR'}

TWENTY_AAs = ['R','H','K','D','E','S','T','N','Q','C','G','P','A','I','L','M','F','W','Y','V']

DEFAULT_COLOR_PALETTE = {'A':'black', 
                         'C':'black',
                         'D':'red',
                         'E':'red',
                         'F':'black',
                         'G':'black',
                         'H':'green', 
                         'I':'black',
                         'K':'blue',
                         'L':'black',
                         'M':'black',
                         'N':'green',
                         'P':'fuchsia',
                         'Q':'green',
                         'R':'blue',
                         'S':'green',
                         'T':'green',
                         'V':'black',
                         'W':'black',
                         'Y':'black'}






# <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
# <>          
# <>          HYDROPHOBICITY DEFINING FUNCTIONS
# <> 
# <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>

# >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
# 
# KYTE-DOOLITTLE SCALES
#
# References
#
# Main hydrophobicity scale
# ...............................
#
# A simple method for displaying the hydropathic character of a protein. 
# Kyte J, Doolittle RF. J Mol Biol. 1982 May 5;157(1):105-32.
#
#
#
# 0 to 1 normalized hydrophobicity scale
# ...............................................
#
# Why are "natively unfolded" proteins unstructured under physiological conditions?
# Valdimir N. Uversky, Joel R. Gillespie, and Anthony L. Frink
# Protines: Structure, function, and genetics 41:415-427 (2000)
#

def get_KD_original():
    """ Function which returns the original KD hydropathy lookup table
    """    
    
    return  {'ILE': 4.5,                
             'VAL': 4.2,
             'LEU': 3.8,
             'PHE': 2.8,
             'CYS': 2.5,
             'MET': 1.9,
             'ALA': 1.8,
             'GLY': -0.4,
             'THR': -0.7,
             'SER': -0.8,
             'TRP': -0.9,
             'TYR': -1.3,
             'PRO': -1.6,
             'HIS': -3.2,
             'GLU': -3.5,
             'GLN': -3.5,
             'ASP': -3.5,
             'ASN': -3.5,
             'LYS': -3.9,
             'ARG': -4.5}

def get_residue_charge():
    """ Function which returns the original KD hydropathy lookup table
    """    
    
    return  {'ILE': 0,                
             'VAL': 0,
             'LEU': 0,
             'PHE': 0,
             'CYS': 0,
             'MET': 0,
             'ALA': 0,
             'GLY': 0,
             'THR': 0,
             'SER': 0,
             'TRP': 0,
             'TYR': 0,
             'PRO': 0,
             'HIS': 0,
             'GLU': -1,
             'GLN': 0,
             'ASP': -1,
             'ASN': 0,
             'LYS': 1,
             'ARG': 1}



def get_KD_shifted():
    """ 
    Function which returns the shifted KD hydropathy lookup table (such that
    it runs from 0 to 9 instead of -4.5 to 4.5)
    """
    
    original  = get_KD_original()
    
    shifted = {}
    for i in original:
        shifted[i] = original[i]+4.5

    return shifted
        

    """ Should look like this!
    {'ALA': 6.3,
    'ARG': 0.0,
    'ASN': 1.0,
    'ASP': 1.0,
    'CYS': 7.0,
    'GLN': 1.0,
    'GLU': 1.0,
    'GLY': 4.1,
    'HIS': 1.3,
    'ILE': 9.0,
    'LEU': 8.3,
    'LYS': 0.6,
    'MET': 6.4,
    'PHE': 7.3,
    'PRO': 2.9,
    'SER': 3.7,
    'THR': 3.8,
    'TRP': 3.6,
    'TYR': 3.2,
    'VAL': 8.7
    }
    """

def get_KD_uversky():
    """
    Returns a 0-to-1 normalized KD scale

    """
    shifted = get_KD_shifted()
    
    uversky = {}
    for i in shifted:
        uversky[i] = shifted[i]/9.0

    return uversky 

           
def get_PPII_Hilser():
    """
    Returns an amino acid dictionary with the PPII propensity of
    each residue as calculated by Elam et al [1] (Taken from [2]). 


    [1] - Elam WA, Schrank TP, Campagnolo AJ, Hilser VJ. Evolutionary 
    conservation of the polyproline II conformation surrounding intrinsically 
    disordered phosphorylation sites. 
    Protein Sci. 2013; 22: 405 - 417. doi: 10.1002/pro.2217 PMID: 23341186

    [2] - Tomasso, M. E., Tarver, M. J., Devarajan, D. & Whitten, S. T. 
    Hydrodynamic Radii of Intrinsically Disordered Proteins Determined 
    from Experimental Polyproline II Propensities. 
    PLoS Comput. Biol. 12, e1004686 (2016).

    """
    return  {'ILE': 0.39,                
             'VAL': 0.39,
             'LEU': 0.24,
             'PHE': 0.17,
             'CYS': 0.25,
             'MET': 0.36,
             'ALA': 0.37,
             'GLY': 0.13,
             'THR': 0.32,
             'SER': 0.24,
             'TRP': 0.25,
             'TYR': 0.25,
             'PRO': 1.00,
             'HIS': 0.20,
             'GLU': 0.42,
             'GLN': 0.53,
             'ASP': 0.30,
             'ASN': 0.27,
             'LYS': 0.56,
             'ARG': 0.38}

    


""""
EMPTY       = {'ILE': 
               'VAL': 
               'LEU': 
               'PHE': 
               'CYS': 
               'MET': 
               'ALA': 
               'GLY': 
               'THR': 
               'SER': 
               'TRP': 
               'TYR': 
               'PRO': 
               'HIS': 
               'GLU': 
               'GLN': 
               'ASP': 
               'ASN': 
               'LYS': 
               'ARG': }
"""


def buildTable():
    return build_amino_acids_skeleton()


def build_amino_acids_skeleton():
    """ 
    The default hydrophobicity is an augmented Kyte-Doolitle where
    0 = most hydrophilic and 9 is most hydrophobic

    This is a weird way of doing this but essentially it means we define
    the charge and hydrophobicity in one place (in separate functions)
    and then construct the skeleton before appending those values to the
    amino acid skeleton.

    The advantage of this is we can add further information to induvidual 
    amino acids. 

    """
    
    # get a dictionary of 3 letter to charge
    charge_Dict = get_residue_charge()

    # get a dictionary of 3 letter to KD hydrophobicity
    KD_Dict     = get_KD_shifted()

    PPII_Dict   = get_PPII_Hilser()
    
    # build the initial skeleton of amnino acids 
    skeleton=[["Alanine",        "ALA", "A"],
            ["Cysteine",       "CYS", "C"],
            ["Aspartic_Acid",  "ASP", "D"],
            ["Glutamic_Acid",  "GLU", "E"],
            ["Phenylalanine",  "PHE", "F"],
            ["Glycine",        "GLY", "G"],
            ["Histidine",      "HIS", "H"],
            ["Isoleucine",     "ILE", "I"],
            ["Lysine",         "LYS", "K"],
            ["Leucine",        "LEU", "L"],
            ["Methionine",     "MET", "M"],
            ["Asparagine",     "ASN", "N"],
            ["Proline",        "PRO", "P"],
            ["Glutamine",      "GLN", "Q"],
            ["Arginine",       "ARG", "R"],
            ["Serine",         "SER", "S"],
            ["Threonine",      "THR", "T"],
            ["Valine",         "VAL", "V"],
            ["Tryptophan",     "TRP", "W"],
            ["Tyrosine",       "TYR", "Y"]]


    # for each residue update with further information
    for res in skeleton:

        # update this residue with the charge value
        res.append(KD_Dict[res[1]])

        # update this residue with the KD hydrophobicity
        res.append(charge_Dict[res[1]])
        
        # update the residue with the PPII content
        res.append(PPII_Dict[res[1]])
    
    return skeleton
    

def update_hydrophobicity(aalist, scale):
    """
    Function which lets you update the hydrophobicity of an amino
    acid to your own scale.

    """
    
    index=0
    for i in aalist:
        i[3] = scale[i[1]]

    return aalist



"""
   !--------------------------------------------------------------------------!
   ! LICENSE INFO:                                                            !
   !--------------------------------------------------------------------------!
   !    This file is part of localCIDER.                                      !
   !                                                                          !
   !    Version 0.1.9                                                         !
   !                                                                          !
   !    Copyright (C) 2014 - 2016                                             !
   !    The localCIDER development team (current and former contributors)     !
   !    Alex Holehouse, James Ahad, Rahul K. Das.                             !
   !                                                                          !
   !    localCIDER was developed in the lab of Rohit Pappu at Washington      !
   !    University in St. Louis. Please see the website for citation          !
   !    information:                                                          !
   !                                                                          !
   !    http://pappulab.github.io/LocalCIDER/                                 !
   !                                                                          !
   !    For more information please see the Pappu lab website:                !
   !                                                                          !
   !    http://pappulab.wustl.edu/                                            !
   !                                                                          !
   !    LocalCIDER is free software: you can redistribute it and/or modify    !
   !    it under the terms of the GNU General Public License as published by  !
   !    the Free Software Foundation, either version 3 of the License, or     !
   !    (at your option) any later version.                                   !
   !                                                                          !
   !    LocalCIDER is distributed in the hope that it will be useful,         !
   !    but WITHOUT ANY WARRANTY; without even the implied warranty of        !
   !    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         !
   !    GNU General Public License for more details.                          !
   !                                                                          !
   !    You should have received a copy of the GNU General Public License     !
   !    along with LocalCIDER.  If not, see <http://www.gnu.org/licenses/>.   !
   !--------------------------------------------------------------------------!
   ! AUTHORSHIP INFO:                                                         !
   !--------------------------------------------------------------------------!
   !                                                                          !
   ! MAIN AUTHOR:   James Ahad and Alex Holehouse                             !
   !                                                                          !
   !--------------------------------------------------------------------------!


   File Description:
   ================

   This is the main file which deals with sequence objects, and is where all
   sequence analysis operations occur. These functions should not be called
   by normal users - if there is functionality or logical carried out in here
   you would like to see in localCIDER please submit a feature request and we
   can integrate these changes into the sequenceparameter API.


REFERENCES
R.K. Das, R.V. Pappu. (2013).
"Conformations of intrinsically disordered proteins are  influenced by
linear sequence distributions of oppositely charged residues."

Proceedings of the National Academy of Sciences USA. 110: 13392-13397

"""

import numpy as np
import random as rng
import time
import copy as cp
import os
import itertools
from backendtools import return_absolute_datafile_path, warning_message, verifyType, status_message, warning_message
from restable import ResTable
from data import aminoacids
import data
from localciderExceptions import SequenceException
from sequenceComplexity import SequenceComplexity


######################

# lookuptable is a global table of AA properties
#
# Hydropathy of amino acids is determined by the KD table
#
lkupTab = ResTable()


class Sequence:
    """
    The sequence class is the main class which holds sequence properties
    and carries out sequence parameter logic operations
    """

    #...................................................................................#
    def __init__(self, seq, dmax=-1, chargePattern=[], validateSeq=False):
        """
        seq = amino acid sequence as a string

        """

        if not verifyType(seq, str):
            raise SequenceException(
                "Must pass a string to a new Sequence object")

        # by default don't validate
        if validateSeq:
            seq = seq.upper()
            seq = self.validateSequence(seq)

        self.seq = seq.upper()
        self.len = len(seq)
        self.chargePattern = chargePattern

        if(chargePattern == []):
            for i in np.arange(0, self.len):
                if(lkupTab.lookUpCharge(self.seq[i]) > 0):
                    chargePattern = np.append(chargePattern, 1)
                elif(lkupTab.lookUpCharge(self.seq[i]) < 0):
                    chargePattern = np.append(chargePattern, -1)
                else:
                    chargePattern = np.append(chargePattern, 0)
            self.chargePattern = chargePattern
        # initializing to prevent extra computational time
        self.dmax = dmax

        # set phosphosites as empty
        self.phosphosites = []

        # set color pallete
        self.set_HTMLColorResiduePalette(aminoacids.DEFAULT_COLOR_PALETTE)

        # build a complexity object (just code encapsulation - no state)
        self.ComplexityObject = SequenceComplexity()

    #...................................................................................#
    def __unicode__(self):
        """ Returns the sequences """
        return self.seq

    #...................................................................................#
    def __str__(self):
        """ Returns the sequences """
        return self.seq

    #...................................................................................#
    def validateSequence(self, seq):

        processed = ""

        AAs = data.aminoacids.ONE_TO_THREE.keys()
        pos = 0
        messageWarned = False

        # for each residue in your protein sequence
        for i in seq:
            pos = pos + 1
            if i not in AAs:

                # if we find whitespace
                if i.isspace():
                    if not messageWarned:
                        # only warn once...
                        status_message("Removing whitespace from sequence")
                        messageWarned = True
                    pass
                # if unexpected residue/character bail
                else:
                    raise SequenceException(
                        "Invalid amino acid [" + str(i) + "] found at position " + str(pos))

            # else append sequence to the processed sequence
            else:
                processed = processed + i

        # determine proline content and warn if over 15%
        prolineContent = float(processed.count("P")) / float(len(processed))
        if prolineContent > 0.15:
            warning_message(
                "This sequence has a proline content of greater than 15%.\nThis may render some analyses [notably kappa and phase diagram predictions] incorrect")

        return processed

    # <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
    #
    #                           GENERAL SEQUENCE PROPERTIES
    #
    # <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>

    #...................................................................................#
    def fraction_disorder_promoting(self):
        """
        Get the fraction of 'disorder promoting residues' in a
        sequence
        """

        D = ['T', 'A', 'G', 'R', 'D', 'H', 'Q', 'K', 'S', 'E', 'P']
        # the O list is redundant but I've included it for good measure!
        O = ['W', 'F', 'Y', 'I', 'M', 'L', 'V', 'N', 'C']

        D_count = 0
        O_count = 0
        for i in self.seq:
            if i in D:
                D_count += 1
            else:
                O_count += 1
        return float(D_count) / len(self.seq)

    def amino_acid_fraction(self):
        AADICT = {'A': 0,
                  'C': 0,
                  'D': 0,
                  'E': 0,
                  'F': 0,
                  'G': 0,
                  'H': 0,
                  'I': 0,
                  'K': 0,
                  'L': 0,
                  'M': 0,
                  'N': 0,
                  'P': 0,
                  'Q': 0,
                  'R': 0,
                  'S': 0,
                  'T': 0,
                  'V': 0,
                  'W': 0,
                  'Y': 0}
        for i in self.seq:
            AADICT[i] += 1

        for i in AADICT:
            AADICT[i] = float(AADICT[i]) / float(len(self.seq))

        return AADICT

    #...................................................................................#
    def countPos(self):
        """ Get the number of positive residues in the sequence """
        return len(np.where(self.chargePattern > 0)[0])

    #...................................................................................#
    def countNeg(self):
        """ Get the number of negative residues in the sequence """
        return len(np.where(self.chargePattern < 0)[0])

    #...................................................................................#
    def countNeut(self):
        """ Get the number of neutral residues in the sequence """
        return len(np.where(self.chargePattern == 0)[0])

    #...................................................................................#
    def Fplus(self):
        """ Get the fraction of positive residues in the sequence """
        return self.countPos() / (self.len + 0.0)

    #...................................................................................#
    def Fminus(self):
        """ Get the fraction of negative residues in the sequence """
        return self.countNeg() / (self.len + 0.0)

    #...................................................................................#
    def FCR(self):
        """ Get the fraction of charged residues in the sequence """
        return (self.countPos() + self.countNeg()) / (self.len + 0.0)

    #...................................................................................#
    def FER(self):
        """ Get the fraction of charged residues in the sequence """
        return (self.countPos() + self.countNeg() + self.seq.count('P')) / (self.len + 0.0)

    #...................................................................................#
    def NCPR(self):
        """ Get the net charge per residue of the sequence """
        return (self.countPos() - self.countNeg()) / (self.len + 0.0)

    #...................................................................................#
    def mean_net_charge(self):
        """ Get the absolute magnitude of the mean net charge """
        return abs(self.NCPR())

    #...................................................................................#
    def kappa(self):
        """
        Return the kappa value, as defined in REF 1 \
        """

        if self.deltaMax() == 0:
            warning_message(
                "The sequence has no charged residues - kappa is not a valid/relevant parameter")
            return -1
        else:

            kappaVal = self.delta() / self.deltaMax()

            # so the heuristics for kappa are good BUT may under estimate
            # deltaMax is some cases. If this is a small deviation then we
            # just set it to 0 because the sequence with the highest delta is probably
            # an sequence-Isomer of the sequence we have. If this deviation is larger,
            # however, it may be indicative of a bug in the code which we
            # should address 
            #
            # It's probably worth pointing out that after > 10 Million sequences 
            # analyzed we're yet to find a bug - May 2016
            if kappaVal > 1.0 and kappaVal < 1.1:
                return 1.0
            else:
                return kappaVal


    #...................................................................................#
    def kappa_proline(self):
        """
        Return the kappa-proline value, as defined in Martin et al. 

        """

        # firstly convert the sequence into a 2 letter alphabet of
        # E/D/R/K/P or 'other'
        newseq=''
        for res in self.seq:
            if res == 'P' or res =='E' or res =='D' or res =='K' or res =='R': 
                newseq=newseq+'E'
            else:
                newseq=newseq+'K'

        # then calculate the new kappa value in this new sequence space and
        # return the value
        augmented_seq = Sequence(newseq)

        return augmented_seq.kappa()


    #...................................................................................#
    def kappa_proline_seq(self):
        """
        Return the kappa-proline sequence, where R/K/D/E/P are X and all other residues
        are O

        """

        # firstly convert the sequence into a 2 letter alphabet of
        # E/D/R/K/P or 'other'
        newseq=''
        for res in self.seq:
            if res == 'P' or res =='E' or res =='D' or res =='K' or res =='R': 
                newseq=newseq+'X'
            else:
                newseq=newseq+'O'

        return newseq


    #...................................................................................#
    def kappa_X(self, grp1, grp2=None):
        """
        Return the user defined kappa value, where residues are subdivided into one
        or two arbitrary groups. If only grp1 is defined then the 20 amino acids are
        divided into grp1 and not grp1, and the patterning is determined for the perfect
        binary system. If grp2 is defined the 20 amino acids are divided into grp1, grp2,
        not grp1 or grp2, and the binary patterning of grp1 and grp2 residues is determined
        in the context of a ternary system.

        As an example, to get the original kappa definition, grouping would be
        grp1 = ['E','D']
        grp2 = ['K','D']

        While for the kappa-proline definition the grouping would be
        grp1 = ['P','E','D','K','R']

        """

        
        ############################################################
        def parse_group(localgrp):
            """ Local function for parsing AA groupings """

            try:
                localgrp = set([x.upper() for x in localgrp])
            except AttributeError:
                raise SequenceException("ERROR: There was a problem when parsing the kappa_X group argument [%s] - this should be a list of of native AA one letter codes" % (localgrp))
                            
            for res in localgrp:
                if res not in aminoacids.TWENTY_AAs:
                    raise SequenceException("When using kappa_X found non-natural amino acid [%s] in grouping"%(res))

            return localgrp
        ############################################################
        
        
        grp1 = parse_group(grp1)

        if grp2:
            grp2 = parse_group(grp2)

        if grp2:
            # firstly convert the sequence into a 3 letter alphabet of
            # grp1, grp2, or 'other'
            newseq=''
            for res in self.seq:
                if res in grp1:
                    newseq=newseq+'E'                
                elif res in grp2:
                    newseq=newseq+'K'
                else:
                    newseq=newseq+'G'

        else:
            # firstly convert the sequence into a 2 letter alphabet of
            # grp1 residue or 'other'
            newseq=''
            for res in self.seq:
                if res in grp1:
                    newseq=newseq+'E'                
                else:
                    newseq=newseq+'K'
            
        # then calculate the new kappa value in this new sequence space and
        # return the value
        augmented_seq = Sequence(newseq)

        return augmented_seq.kappa()


    #...................................................................................#
    def phasePlotRegion(self):
        """ Returns the IDP diagram of states [REF 1] region based on the FCR/NCPR
            rules. Possible return values are;


            1 +-----------------------+---------------+
              |                     +                 |
              |                   +                   |
              |      4          +                     |
        F     |               +                       |
        R     |             +                         |
        A     |           +                           +
        C     |         +                           + |
        T     |       +                           +   |
        .     |     +            3              +     |
          0.5 |   +                           +       |
        N     | +                           +         |
        E     +                           +           |
        G     | +                       +             |
              + 2 +                   +               |
        R     | +   +               +       4         |
        E     |   + 2 +           +                   |
        S     |     +   +       +                     |
              | 1     + 2 +   +                       |
           0  +---------+---+-------------------------+

              0                  0.5                  1
                    FRACTION OF POSITIVE RESIDUES


        1) Weak polyampholytes and polyelectrolyte
        2) Janus sequences
        3) Strong polyampholytes
        4) Strong polyelectrolytes

        """
        fcr = self.FCR()
        ncpr = self.NCPR()

        # if we're in region 1
        if(fcr < .25):
            return 1

        # if we're in region 2
        elif(fcr >= .25 and fcr <= .35):
            return 2

        # if we're in region 3
        elif(fcr > .35 and abs(ncpr) < 0.35):
            return 3

        # if we're in region 4 or 5
        elif(self.Fplus() > 0.35):
            if self.Fminus() > 0.35:
                raise SequenceException(
                    "Algorithm bug when coping with phase plot regions")
            return 5

        elif(self.Fminus() > 0.35):
            return 4

        else:  # This case is impossible but here for completeness\
            raise SequenceException(
                "Found inaccessible region of phase diagram. Numerical error")

    #...................................................................................#
    def phasePlotAnnotation(self):
        """ Return the string annotation for the diagram of states region
            which this sequence falls into
        """
        region = self.phasePlotRegion()
        if(region == 1):
            return 'Globule/Tadpole'
        elif(region == 2):
            return 'Boundary Region'
        elif(region == 3):
            return 'Coils,Hairpins and Chimeras'
        elif(region == 4):
            return 'Negatively Charged Swollen Coils'
        elif(region == 5):
            return 'Positively Charged Swollen Coils'
        else:
            return 'ERROR, NOT A REAL REGION'

    #...................................................................................#
    def meanHydropathy(self):
        """ Return the mean hydropathy value for the sequence
        """
        ans = 0
        for i in xrange(0, self.len):
            ans += lkupTab.lookUpHydropathy(self.seq[i]) / self.len
        return ans

    #...................................................................................#
    def uverskyHydropathy(self):
        """
        Return the mean hydropathy as calculated by Uversky (window size of
        five, with normalized Kyte-Doolitle scale.

        This means we can use different hydropathy scales if we choose, but
        it won't break the Uversky plots
        """

        normalizedKD = data.aminoacids.get_KD_uversky()
        translate = data.aminoacids.ONE_TO_THREE

        ans = 0
        for idx in xrange(0, self.len):
            ans += normalizedKD[translate[self.seq[idx]]] / self.len

        return ans

    #...................................................................................#
    def cumMeanHydropathy(self):
        """ Return a vector of the average cumulative hydropathy. i.e.

        [ vector of cumulative hydropathy ] / [ vector of sequence position ]

        This allows you to examine how hydropathy changes through the sequence
        """
        ans = [lkupTab.lookUpHydropathy(self.seq[0])]
        for i in xrange(1, self.len):
            ans.append(ans[i - 1] + lkupTab.lookUpHydropathy(self.seq[i]))
        ans /= (np.arange(0, self.len) + 1)
        return ans
        
    #...................................................................................#
    def FPPII_chain(self):
        """ 
        Returns the overal chain's average PPII propensity as defined by Elam et al[1].

        [1] - Elam WA, Schrank TP, Campagnolo AJ, Hilser VJ. Evolutionary 
        conservation of the polyproline II conformation surrounding intrinsically 
        disordered phosphorylation sites. 
        Protein Sci. 2013; 22: 405- 417. doi: 10.1002/pro.2217 PMID: 23341186
        
        """
        
        # calculate the total PPII sum for the sequence
        total = 0
        for i in xrange(0, self.len):
            total = total + lkupTab.lookUpPPII(self.seq[i])

        # normalize by the sequence length
        return (total/float(self.len))
        
    #...................................................................................#
    def linearDistOfNCPR(self, bloblen):
        """
        Returns a np vertical stack object showing the NCPR over
        blob-sized regions along the sequence
        """

        # detemrine the number of blobs of length blpblen in the sequence
        nblobs = self.len - bloblen + 1

        # determine the flanking positions over which we don't calculate 
        # NCPR values BUT still want to include to build a profile which is
        # the correct length. 
        flank = int(bloblen/2)
        
        if 2*flank+nblobs == self.len:
            flank_start = flank
            flank_end   = flank
        else:
            flank_start = flank - 1
            flank_end   = flank 
            
        # initialize an empty list of blobs
        blobncpr = [0] * nblobs

        # for each overlapping blob in the sequence calculate the NCPR
        for i in np.arange(0, nblobs):
            blob = self.chargePattern[i:(i + bloblen)]
            bpos = len(np.where(blob > 0)[0])
            bneg = len(np.where(blob < 0)[0])
            blobncpr[i] = (bpos - bneg) / (bloblen + 0.0)

        return np.vstack((np.arange(1, self.len + 1), [0]*flank_start + blobncpr + [0]*flank_end))

    

    #...................................................................................#
    def linearDistOfFCR(self, bloblen):
        """
        Returns an np vertical stack object showing the FCR over
        blob-sized regions along the sequence
        """

        self.__check_window_to_length(bloblen)

        nblobs = self.len - bloblen + 1

        # determine the flanking positions over which we don't calculate 
        # FCR values BUT still want to include to build a profile which is
        # the correct length. 
        flank = int(bloblen/2)

        # if bloblen is odd
        if 2*flank+nblobs == self.len:
            flank_start = flank
            flank_end   = flank 
        else:
            flank_start = flank - 1
            flank_end   = flank 

        blobfcr = [0] * nblobs
        
        # for each overlapping blob in the sequence calculate the FCR
        for i in np.arange(0, nblobs):
            blob = self.chargePattern[i:(i + bloblen)]
            bpos = len(np.where(blob > 0)[0])
            bneg = len(np.where(blob < 0)[0])
            blobfcr[i] = (bpos + bneg) / (bloblen + 0.0)

        return np.vstack((np.arange(1, self.len + 1), [0]*flank_start + blobfcr + [0]*flank_end))

    #...................................................................................#
    def linearDistOfSigma(self, bloblen):
        """
        Returns an np vertical stackobject showing how the "sigma" parameter
        varies over blob-sized regions along the sequence
        """

        self.__check_window_to_length(bloblen)

        nblobs = self.len - bloblen + 1

        # determine the flanking positions over which we don't calculate 
        # sigma values BUT still want to include to build a profile which is
        # the correct length. 
        flank = int(bloblen/2)
        
        if 2*flank+nblobs == self.len:
            flank_start = flank
            flank_end   = flank
        else:
            flank_start = flank - 1
            flank_end   = flank 

        blobsig = [0] * nblobs

        # for each overlapping blob in the sequence calculate the sigma
        # parameter
        for i in np.arange(0, nblobs):
            blob = self.chargePattern[i:(i + bloblen)]
            bpos = len(np.where(blob > 0)[0])
            bneg = len(np.where(blob < 0)[0])

            bncpr = (bpos - bneg) / (bloblen + 0.0)
            bfcr = (bpos + bneg) / (bloblen + 0.0)

            if(bfcr == 0):
                bsig = 0
            else:
                bsig = bncpr**2 / bfcr
            blobsig[i] = bsig

        return np.vstack((np.arange(1, self.len + 1), [0]*flank_start + blobsig + [0]*flank_end))
        
    #...................................................................................#
    def linearDistOfHydropathy(self, bloblen):
        """
        Returns an np vertical stackobject showing how the 0 to 1 normalized hydropathy
        varies over blob-sized regions along the sequence
        """
        
        self.__check_window_to_length(bloblen)

        nblobs = self.len - bloblen + 1

        # determine the flanking positions over which we don't calculate 
        # hydropathy values BUT still want to include to build a profile which is
        # the correct length. 
        flank = int(bloblen/2)
        
        if 2*flank+nblobs == self.len:
            flank_start = flank
            flank_end   = flank
        else:
            flank_start = flank - 1
            flank_end   = flank         

        blobhydro = [0] * nblobs

        # construct a vector of the hydropathy values of each residue in
        # in the sequence (using the Uversky-normalized KD scale which runs from 0 to 1
        # with 1 being the most hydrophobic.

        # get Uversky hydropathy lookup table
        KDU = aminoacids.get_KD_uversky()

        hydrochain = []
        for i in self.seq:
            hydrochain.append(KDU[aminoacids.ONE_TO_THREE[i]])

        # for each overlapping blob in the sequence calculate the hydropathy
        for i in np.arange(0, nblobs):
            blob = hydrochain[i:(i + bloblen)]
            blobhydro[i] = sum(blob) / float(bloblen)

        return np.vstack((np.arange(1, self.len + 1), [0]*flank_start + blobhydro + [0]*flank_end))

    #...................................................................................#
    def linearDistOfHydropathy_2(self, bloblen):
        """
        Returns an np vertical stackobject showing how the 0 to 9 normallized hydropathy
        varies over blob-sized regions along the sequence
        """

        self.__check_window_to_length(bloblen)
        
        nblobs = self.len - bloblen + 1

        flank = int(bloblen/2)
        
        if 2*flank+nblobs == self.len:
            flank_start = flank
            flank_end   = flank
        else:
            flank_start = flank - 1
            flank_end   = flank

        blobhydro = [0] * nblobs

        # construct a vector of the hydropathy values of each residue in
        # in the sequence (using the re-set KD scale which runs from 0 to 9
        # with 9 being the most hydrophobic. This scaling may be changed
        # in future versions...
        hydrochain = [lkupTab.lookUpHydropathy(res) for res in self.seq]

        # for each overlapping blob in the sequence calculate the hydropathy
        for i in np.arange(0, nblobs):
            blob = hydrochain[i:(i + bloblen)]
            blobhydro[i] = sum(blob)

        return np.vstack((np.arange(1, self.len + 1), [0]*flank_start + blobhydro + [0]*flank_end))

    #...................................................................................#
    def linearDenistyOfAAs(self,
                                  bloblen,
                                  targetAAs):
        """
        Function which returns a np.vertical stackobject showing the 0 to 1 normalized density
        of the AAs in the targetAAs list. 


        bloblen defines the window size used, while targetAAs should be a list of one or 
        more amino acids of interest (upper case).

        """

        self.__check_window_to_length(bloblen)
        
        nblobs = self.len - bloblen + 1

        flank = int(bloblen/2)
        
        if 2*flank+nblobs == self.len:
            flank_start = flank
            flank_end   = flank
        else:
            flank_start = flank - 1
            flank_end   = flank

        blob_density = [0] * nblobs

        # construct a binary list where residues in the targetAA list
        # are set to 1 and all others are set to 0 
        target_seq=[]
        for res in self.seq:
            if res in targetAAs:
                target_seq.append(1.0)
            else:
                target_seq.append(0.0)
            
        # construct a vector of the hydropathy values of each residue in
        # in the sequence (using the re-set KD scale which runs from 0 to 9
        # with 9 being the most hydrophobic. This scaling may be changed
        # in future versions...

        # for each overlapping blob in the sequence calculate the hydropathy
        for i in np.arange(0, nblobs):
            blob = target_seq[i:(i + bloblen)]
            blob_density[i] = sum(blob)/float(bloblen)

        return np.vstack((np.arange(1, self.len + 1), [0]*flank_start + blob_density + [0]*flank_end))
    
    #...................................................................................#
    def linearCompositions(self,
                           bloblen,
                           grps=[]):

        """
        Function which returns an n by X matrix showing the density of different groups of amino
        acids running along the sequence. The amino acid group can be defined by the user by providing
        a list of lists, where each sub-list contains amino acids for a distinct group. Alternativly, the
        default grouping provides 7 groups which are

        acidic    (E and D)
        basic     (K and R)
        charged   (E, D, K, and R)
        polar     (Q, N, S, T, G, C, and G)
        aliphatic (L, M, I, and V)
        aromatic  (F, Y, and W)
        proline   (P)
        alanine   (A)

        """        
        
        # if we've defined our own groups (NOTE no sanity checking...)
        if len(grps) > 0:

            # then skip ahead...
            pass

        # else use the standard AA grouping
        else:                
            grps.append(['E','D'])                     # acidic
            grps.append(['R','K'])                     # basic
            grps.append(['R','K','E','D'])             # charged
            grps.append(['Q','N','S','T','G','H'])     # polar
            grps.append(['A','L','M','I','V'])         # hydrophobic
            grps.append(['F','Y','W'])                 # aromatic
            grps.append(['P'])                         # proline
            
        # first create a np vector object with the linear density of the
        # first group
        tmp = self.linearDenistyOfAAs(bloblen, grps[0])
        density = tmp[1]
        
        # for for each remaining group just keep adding to it
        for group in grps[1:]:
            tmp = self.linearDenistyOfAAs(bloblen, group)
            density = np.vstack((density, tmp[1]))

        # finally give the         
        return (tmp[0], density)
            
            
        
    # <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
    #
    #                         SEQUENCE COMPLEXITY FUNCTIONS
    #
    # <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>

    #...................................................................................#
    def get_linear_WF_complexity(
            self,
            alphabetSize=20,
            userAlphabet={},
            windowSize=10,
            stepSize=1):
        """
        Returns the Wooton-Federhen (WF) vectorial complexity using a sliding window approach

        """
        
        self.__check_window_to_length(windowSize)

        return self.ComplexityObject.get_WF_complexity(
            self.seq, alphabetSize, userAlphabet, windowSize, stepSize)

    #...................................................................................#
    def get_linear_LC_complexity(
            self,
            alphabetSize=20,
            userAlphabet={},
            windowSize=10,
            stepSize=1,
            wordSize=3):
        """
        Returns the Linguistic Complexity (LC) vectorial complexity using a sliding window approach

        """

        self.__check_window_to_length(windowSize)

        return self.ComplexityObject.get_LC_complexity(
            self.seq, alphabetSize, userAlphabet, windowSize, stepSize, wordSize)

    #...................................................................................#
    def get_linear_LZW_complexity(
            self,
            alphabetSize=20,
            userAlphabet={},
            windowSize=10,
            stepSize=1):
        """
        Returns the Lempel-Ziv-Welch (LZW) vectorial complexity using a sliding window approach.

        """

        self.__check_window_to_length(windowSize)

        return self.ComplexityObject.get_LZW_complexity(
            self.seq, alphabetSize, userAlphabet, windowSize, stepSize)


    #...................................................................................#
    def get_reducedAlphabetSequence(self, alphabetSize=20, userAlphabet={}):
        """
        Returns the amino acid sequence after being translated to a reduced alphabet

        """
        return self.ComplexityObject.reduce_alphabet(
            self.seq, alphabetSize, userAlphabet)

    # <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
    #
    #                         KAPPA CALCULATING FUNCTIONS
    #
    # <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
    #
    # The functions in the following section focus on parameters involved in the kappa
    # calculation.
    #


    #...................................................................................#
    def sigma(self):
        """ Returns the sigma value for a sequence

           \sigma = \dfrac{NCPR^2}{FCR}

           When the sequence has one or more charged residues
           sigma = (NCPR^2)/FCR

           When the sequence has no charged residues
           sigma = 0
        """
        if(self.countNeut() == self.len):
            return 0
        else:
            return self.NCPR()**2 / self.FCR()

    #...................................................................................#
    def deltaForm(self, bloblen):
        """ Calculate the delta value as defined in REF 1
        """

        sigma = self.sigma()
        nblobs = self.len - bloblen + 1
        ans = 0

        for i in xrange(0, nblobs):

            # get the blob charge pattern list
            blob = self.chargePattern[i:(i + bloblen)]

            # calculate a bunch of parameters for the blob
            # with the blob sigma value being the ultimate
            # goal

            bpos = np.where(blob > 0)[0].size
            bneg = np.where(blob < 0)[0].size

            bncpr = (bpos - bneg) / (bloblen + 0.0)
            bfcr = (bpos + bneg) / (bloblen + 0.0)

            if(bfcr == 0):
                bsig = 0
            else:
                bsig = bncpr**2 / bfcr

            # calculate the square deviation of the
            # blob sigma from the sequence sigma and
            # weight by the number of blobs in the sequence
            ans += (sigma - bsig)**2 / nblobs

        return ans

    #...................................................................................#
    def delta(self):
        """
        Return the average delta value from blobs of size 5 and 6, as
        arrived upon as being optimal in REF 1
        """

        return (self.deltaForm(5) + self.deltaForm(6)) / 2

    #...................................................................................#
    def deltaMax(self):
        """
        Return the maximum possible delta value for the sequence
        """

        # If this has been computed already, then return it
        if(self.dmax != -1):
            return self.dmax
        elif(self.FCR() == 0):
            self.dmax = 0

        #################################################################
        # FIRST computational trick - if only positive or negative
        elif self.countPos() == 0 or self.countNeg() == 0:
            nneuts = self.countNeut()

            # construct a neutral  block
            neutralBlock = '0' * nneuts

            # construct a charged block
            if self.countPos() == 0:
                # make a negative charge block
                chargedBlock = '-' * self.countNeg()
                chargeV = "-"
                ncharge = self.countNeg()

            else:
                # make a positive charge block
                chargedBlock = '+' * self.countPos()
                chargeV = "+"
                ncharge = self.countPos()

            # if the charge block is shorter than the neutral block
            if len(neutralBlock) > len(chargedBlock):

                for position in xrange(0, (self.len - ncharge) + 1):

                    setupSequence = position * "0" + \
                        chargedBlock + "0" * (nneuts - position)

                    if not len(setupSequence) == self.len:
                        raise SequenceException(
                            "Error in DeltaMax calculation")

                    nseq = Sequence(setupSequence)

                    # update self.dmax if relevant
                    self.dmax = max([nseq.delta(), self.dmax])
            # if the neutral block is shorter
            else:

                for position in xrange(0, (self.len - nneuts) + 1):

                    setupSequence = position * chargeV + \
                        neutralBlock + chargeV * (ncharge - position)

                    if not len(setupSequence) == self.len:
                        raise SequenceException(
                            "Error in DeltaMax calculation")

                    nseq = Sequence(setupSequence)

                    # update self.dmax if relevant
                    self.dmax = max([nseq.delta(), self.dmax])

        #################################################################
        # Second computational trick (Maximum Charge Separation)
        # If we have no neutral residues this is easy
        elif(self.countNeut() == 0):

            nNeg = self.countNeg()
            nPos = self.countPos()

            posBlock = "+" * nPos
            negBlock = "-" * nNeg

            if len(posBlock) > len(negBlock):
                for position in xrange(0, (self.len - nNeg) + 1):
                    setupSequence = position * "+" + \
                        negBlock + "+" * (nPos - position)

                    if not len(setupSequence) == self.len:
                        raise SequenceException(
                            "Error in DeltaMax calculation")

                    nseq = Sequence(setupSequence)

                    # update self.dmax if relevant
                    self.dmax = max([nseq.delta(), self.dmax])
            else:
                for position in xrange(0, (self.len - nPos) + 1):
                    setupSequence = position * "-" + \
                        posBlock + "-" * (nNeg - position)

                    if not len(setupSequence) == self.len:
                        raise SequenceException(
                            "Error in DeltaMax calculation")

                    nseq = Sequence(setupSequence)

                    # update self.dmax if relevant
                    self.dmax = max([nseq.delta(), self.dmax])

        #################################################################
        # Third computational trick (Maximization of # of Charged Blobs)
        # relevant if we have 18 or more neutral residues
        elif(self.countNeut() >= 18):
            nneuts = self.countNeut()

            # construct positive and negative blocks

            posBlock = '+' * self.countPos()
            negBlock = '-' * self.countNeg()

            # There are three regions where you can have neutral residues
            # to maximize delta: start/middle/end
            # This set of loops iterates through the various relevant combinations
            # of neutral residues at those locations. Importantly, this depends
            # on a blob length of 5-6!

            for startNeuts in xrange(0, 7):
                for endNeuts in xrange(0, 7):
                    setupSequence = ''
                    midBlock = ''
                    endBlock = ''

                    setupSequence = '0' * startNeuts
                    endBlock = '0' * endNeuts
                    midBlock = '0' * (nneuts - startNeuts - endNeuts)

                    # now we construct *an* optimal sequence based on the start/mid/end
                    # permutation of neutral residues
                    setupSequence += posBlock
                    setupSequence += midBlock
                    setupSequence += negBlock
                    setupSequence += endBlock
                    nseq = Sequence(setupSequence)

                    # update self.dmax if relevant
                    self.dmax = max([nseq.delta(), self.dmax])

        #################################################################
        # Fourth computational trick (Search through set of sequences that fit
        # DMAX pattern)
        else:
            posBlock = ''
            negBlock = ''
            nneuts = self.countNeut()

            # construct positive and negative blocks
            posBlock = '+' * self.countPos()
            negBlock = '-' * self.countNeg()

            # iterate through different permutations where
            #  the size of the middle number of neutral residues varies
            for midNeuts in xrange(0, nneuts + 1):
                midBlock = '0' * midNeuts

                for startNeuts in xrange(0, nneuts - midNeuts + 1):
                    setupSequence = ''

                    # construct some permutation of the sequence
                    setupSequence = '0' * startNeuts

                    setupSequence += posBlock
                    setupSequence += midBlock
                    setupSequence += negBlock

                    setupSequence = setupSequence + '0' * \
                        (nneuts - startNeuts - midNeuts)

                    nseq = Sequence(setupSequence)

                    # update self.dmax if relevant
                    self.dmax = max([nseq.delta(), self.dmax])

        return self.dmax

    #...................................................................................#
    def swapRes(self, index1, index2):
        """
        Causes the two residues indexed by index1 and index2 in the sequence
        to be swapped. This returns a new sequence object with pre-calculated dmax
        """

        if(index1 == index2):
            return Sequence(self.seq)

        # we want index 2 to be greater than index 1
        elif(index2 < index1):
            index1, index2 = index2, index1
        else:
            pass

        # create a new tempseq as a charlist and then
        # swap the characters and positions index1 and index2
        tempseq = list(self.seq)
        tempseq[index1], tempseq[index2] = tempseq[index2], tempseq[index1]

        charge1 = self.chargePattern[index1]
        charge2 = self.chargePattern[index2]

        # creates a totally new array with new values - do we need this?
        tempChargeSeq = cp.deepcopy(self.chargePattern)
        tempChargeSeq[index1] = charge2
        tempChargeSeq[index2] = charge1

        # returns a new sequence with dmax and the chargeSequence already
        # generated
        return Sequence(''.join(tempseq), self.dmax, tempChargeSeq)

    #...................................................................................#
    def swapRandChargeRes(self, frozen=set()):
        """ Function which randomly selects two residues and swaps them if that
            swap would change the kappa value
        """

        # get a random number
        rand = rng.Random()
        rand.seed(time.time())

        # determine the indices from which we can swap

        # (i.e. all positive indices which do not overlap with the set of frozen
        # residues)
        posInd = set(np.where(self.chargePattern > 0)[0]) - frozen
        negInd = set(np.where(self.chargePattern < 0)[0]) - frozen
        neutInd = set(np.where(self.chargePattern == 0)[0]) - frozen

        if(len(neutInd) == 0):
            if(len(posInd) == 0 or len(negInd) == 0):
                status_message(
                    'swap will not change kappa, only one charge type in sequence')
                return self
            else:
                chargeType = [1, 2]
        elif(len(negInd) == 0):
            if(len(posInd) == 0 or len(neutInd) == 0):
                status_message(
                    'swap will not change kappa, only one charge type in sequence')
                return self
            else:
                chargeType = [1, 3]
        elif(len(posInd) == 0):
            if(len(negInd) == 0 or len(neutInd) == 0):
                status_message(
                    'swap will not change kappa, only one charge type in sequence')
                return self
            else:
                chargeType = [2, 3]
        else:
            chargeType = rand.sample([1, 2, 3], 2)

        if(chargeType[0] == 1):
            swapPair1 = rand.sample(posInd, 1)
        elif(chargeType[0] == 2):
            swapPair1 = rand.sample(negInd, 1)
        elif(chargeType[0] == 3):
            swapPair1 = rand.sample(neutInd, 1)

        if(chargeType[1] == 1):
            swapPair2 = rand.sample(posInd, 1)
        elif(chargeType[1] == 2):
            swapPair2 = rand.sample(negInd, 1)
        elif(chargeType[1] == 3):
            swapPair2 = rand.sample(neutInd, 1)
        return self.swapRes(swapPair1[0], swapPair2[0])

    #...................................................................................#
    def full_shuffle(self, frozen=set()):
        """
           Function which totally shuffles a sequences, but keeps the positions
           in the frozen set in their position

        """

        moveable_indicies = set(np.arange(0, self.len)) - set(frozen)

        lookup = {}
        index = 0
        for i in self.seq:
            lookup[index] = i
            index = index + 1

        # lookup now corresponds to an index->AA lookupyable with lookup
        # indexed

        newseq = list(moveable_indicies)
        rand = rng.Random()
        rand.seed(time.time())

        rand.shuffle(newseq)
        sequencelist = list(self.seq)

        new_seq = []

        for i in xrange(0, self.len):
            if i in frozen:
                new_seq.append(lookup[i])
            else:
                new_seq.append(lookup[newseq.pop()])

        return Sequence("".join(new_seq), self.dmax)

    # <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
    #
    #                         PHOSPHORYLATION RELATED FUNCTIONS
    #
    # <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
    #
    # The functions below focus on phosphorylation and the like
    #

    #...................................................................................#
    def setPhosPhoSites(self, listOfPsites):
        """
        Set one or more sites on your sequence which can be phosphorylated. Note that
        this indexes from 1 (like all of bioinformatics) and not from 0 (like all of
        computer science).

        i.e. "KKKYKKK" the Y here is at position 4

        Internally we do translate to indexing from 0, but this is not something you
        should have to worry about

        Note that all data validation for the phosphosite list is done in this function.

        """

        # if we passed a single value not in a list then convert to a list of length
        # 1
        if isinstance(listOfPsites, int):
            tmp = listOfPsites
            listOfPsites = []
            listOfPsites.appned(tmp)

        # evaluate proposed phosphosites
        for site in listOfPsites:

            # check we can convert to an integer!
            site = int(site)

            # python indexes from 0 but humans from 1
            idx = site - 1

            # if we're outside our sequence
            if idx >= len(self.seq) or idx < 0:
                warning_message("Proposed phosphosite (" + str(idx + 1) +
                                " is outside sequence range. Skipping...")
                pass

            # grab the residue letter from the sequence
            res = self.seq[idx]

            status_message("Setting " + res + str(idx + 1))

            if res not in ["S", "T", "Y"]:
                # we skip it if it seems like an unphosphorylatable residue
                warning_message(
                    'Position ' +
                    str(site) +
                    ' in sequence is a non phosphorylatable residue [' +
                    str(res) +
                    ']')
            else:
                if idx in self.phosphosites:
                    # don't add the same residue twice, but no need to warn
                    # about it
                    pass
                else:
                    # let's add that bad-boy!
                    self.phosphosites.append(idx)

    #...................................................................................#
    def calculateNumberDifferentPhosphoStates(self):
        """ Function which returns the number of different phosphorylation
            states assuming each phosphosite can be phosphorylated independently
            of each other.

            This can be useful if you want to estimate how long a complete
            phoshostate distribution calculation might take (each calc is about
            0.2-1 second, depending on your CPU)
        """
        return np.power(2, len(self.phosphosites))

    #...................................................................................#
    def clear_phosphosites(self):
        """ Function which zeros out all the stored phosphosite data """
        self.phosphosites = []

    #...................................................................................#
    def calculateKappaDistOfPhosphoStates(self):
        """ Function which, using the previously defined possible phosphosites, calculates the cloud
            of possible kappa values for all combination of phosphorylation. This is a relativly long
            operation due to the fact that the FULL kappa must be recalculated everytime (i.e. delta max
            must be recomputed each time, unlike with sequence permutants where DMAX remains constant).

            To aid with this, every 50 calculations the progress is printed

        """
        ##
        #----------------------------------------------------------------------
        # local function
        def print_progress(count, total):
            if (count % 50) == 0:
                status_message("Done " + str(count) + " of " + str(total))

        num_calcs = self.calculateNumberDifferentPhosphoStates()
        #----------------------------------------------------------------------

        # itertools.product("01"..) produces all variants of a list where each
        # element is either 0 or 1 using an iterator
        phosphokappa = []
        count = 0
        for phosphostatus in itertools.product(
                "01", repeat=len(self.phosphosites)):
            newseq = list(self.seq)

            indx = 0

            # look over each element in our phosphosite on/off list
            for i in phosphostatus:
                # if that element is ON
                if int(i) == 1:
                    # set the AA at that position to a negative residue (we use E but
                    # could be D)
                    newseq[self.phosphosites[indx]] = "E"
                indx = indx + 1
            # now we've replaced some number of T/Y/R with E representing a different
            # phosphostate
            newseq = "".join(newseq)
            newseqObj = Sequence(newseq)

            # now add that new kappa to the list, update, and repeat!
            phosphokappa.append(
                (newseqObj.kappa(),
                 newseqObj.Fplus(),
                 newseqObj.Fminus(),
                 newseqObj.FCR(),
                 newseqObj.NCPR(),
                 newseqObj.meanHydropathy(),
                 phosphostatus))

            print_progress(count, num_calcs)
            count = count + 1

        return phosphokappa

    #...................................................................................#
    def get_phosphosites(self):
        """ Returns the list of phosphosites (indexed from 1
            not zero)
        """
        newSites = []
        for i in self.phosphosites:
            newSites.append(i + 1)
        return newSites

    #...................................................................................#
    def get_phosphosequence(self):
        """
        Returns a sequence with all phosphorylated residues converted to E.

        Phosphorylated residues defined by the phosphosites object variable

        """

        if len(self.phosphosites) == 0:
            warning_message(
                "No phosphosites defined - phosphosequence will be equivalent to the unphosphorylated sequence")

        # first defined the empty sequence
        pseq = ""
        idx = 0

        # for each position if that residue is phosphorylatable set it to 'E' instead
        # of the actual Y/S/T
        for i in self.seq:
            if idx in self.phosphosites:

                # extra level of checking!
                if i not in ["S", "Y", "T"]:
                    raise SequenceException(
                        "In get_phosphosequence - trying to replace non-phsophrylatable residue with GLU")

                pseq = pseq + "E"
            else:
                pseq = pseq + self.seq[idx]

            idx = idx + 1
        return pseq

    #...................................................................................#
    def kappa_at_maxPhos(self):
        """
        Returns the kappa if all the phosphosites
        were phosphorylated
        """

        # no phosphosites defined
        if len(self.phosphosites) == 0:
            return self.kappa()
        else:
            newseq = list(self.seq)
            for pos in self.phosphosites:
                newseq[pos] = "E"
            newseq = "".join(newseq)
            newseqObj = Sequence(newseq)

            return newseqObj.kappa()

    #...................................................................................#
    def get_STY_residues(self):
        """
        Where are all the Ser/Thr/Tyr residues in your
        sequence.

        Note we return positions which index from 1 not from 0

        """

        sites = []
        idx = 1
        for i in self.seq:
            if i in ["Y", "S", "T"]:
                sites.append(idx)
            idx = idx + 1
        return sites

    # <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
    #
    #                               FORMATTING FUNCTION
    #
    # <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>

    #...................................................................................#
    def toString(self):
        """


        """
        s = "%i\t%3.5f\t%3.5f\t%3.5f\t%3.5f\t%3.5f\t%3.5f\t%3.5f\t%3.5f\t%3.5f" % (self.len, self.Fminus(), self.Fplus(
        ), self.FCR(), self.NCPR(), self.sigma(), self.delta(), self.deltaMax(), self.kappa(), self.meanHydropathy())
        return s

    #...................................................................................#
    def set_HTMLColorResiduePalette(self, colorDict):
        """
        Function which lets you userdefine the color pallete being used with a dictionary of [AA]="color" mappings where
        color must be one of the 17 key HTML colours;

        ['aqua', 'black', 'blue', 'fuchsia', 'gray', 'green', 'lime', 'maroon', 'navy', 'olive', 'orange', 'purple',
        'red', 'silver', 'teal', 'white', 'yellow']

        """

        valid = {}

        # for each one letter amino acid code
        for i in aminoacids.ONE_TO_THREE:

            # check all the amino acids were there
            if i not in colorDict:
                raise SequenceException(
                    "When trying to set amino acid color palette the amino acid " +
                    str(i) +
                    " was missing from the input dictionary")

            # check if the color is real
            if colorDict[i] not in [
                    'aqua',
                    'black',
                    'blue',
                    'fuchsia',
                    'gray',
                    'green',
                    'lime',
                    'maroon',
                    'navy',
                    'olive',
                    'orange',
                    'purple',
                    'red',
                    'silver',
                    'teal',
                    'white',
                    'yellow']:
                raise SequenceException(
                    "When trying to set amino acid color palette the color " + str(
                        colorDict[i]) + " was used, which is not one of the 17 standard HTML colours")

            # if we get here update the 'valid' dictionary
            valid[i] = colorDict[i].lower()

        # if we got here we got through the full 20 amino acids succesfully
        self.aminoAcidColorMap = {}
        for i in valid:
            self.aminoAcidColorMap[i] = valid[i]

    #...................................................................................#
    def get_HTMLColorString(self):
        """
        Function which creates a <p> </p> wrapped HTML string with the sequence colored according to the defined
        aminoAcidColorMap.

        """
        colorString = '<p style="font-family:Courier;">'
        count = -1
        for residue in self.seq:
            count = count + 1
            if(np.mod(count, 10) == 0):
                colorString = colorString + " "
            if(np.mod(count, 50) == 0):
                colorString = colorString + "<br>"

            color = self.aminoAcidColorMap[residue]

            colorString = '%s<span style="color:%s">%s</span>' % (
                colorString, color, residue)
        colorString = colorString + "</p>"
        return colorString

    #...................................................................................#
    def toFileString(self):
        s = "Sequence   :\t%s\n\n" % (self.seq)
        s += "N          :\t%i\n" % (self.len)
        s += "f-         :\t%3.5f\n" % (self.Fminus())
        s += "f+         :\t%3.5f\n" % (self.Fplus())
        s += "FCR        :\t%3.5f\n" % (self.FCR())
        s += "NCPR       :\t%3.5f\n" % (self.NCPR())
        s += "Kappa      :\t%3.5f\n" % (self.kappa())
        s += "Sigma      :\t%3.5f\n" % (self.sigma())
        s += "Delta      :\t%3.5f\n" % (self.delta())
        s += "Max Delta  :\t%3.5f\n" % (self.deltaMax())
        s += "Hydropathy :\t%3.5f\n\n" % (self.meanHydropathy())
        s += "Phase Plot Region: %i\n" % (self.phasePlotRegion())
        s += "Phase Plot Annotation: %s\n" % (self.phasePlotAnnotation())
        return s
    

    #...................................................................................#
    def __check_window_to_length(self, bloblen):
        """
        Check to ensure that a window size is not greater than the sequence length
        which will cause issues...


        """
        if len(self.seq) < bloblen:
            raise SequenceException('Trying to use a window/blob size of %i but the sequence is only %i residues in length! Window size must be < the sequence length.' % (bloblen, len(self.seq)))

                                



"""
   !--------------------------------------------------------------------------!
   ! LICENSE INFO:                                                            !
   !--------------------------------------------------------------------------!
   !    This file is part of localCIDER.                                      !
   !                                                                          !
   !    Version 0.1.9                                                         !
   !                                                                          !
   !    Copyright (C) 2014 - 2016                                             !
   !    The localCIDER development team (current and former contributors)     !
   !    Alex Holehouse, James Ahad, Rahul K. Das.                             !
   !                                                                          !
   !    localCIDER was developed in the lab of Rohit Pappu at Washington      !
   !    University in St. Louis. Please see the website for citation          !
   !    information:                                                          !
   !                                                                          !
   !    http://pappulab.github.io/localCIDER/                                 !
   !                                                                          !
   !    For more information please see the Pappu lab website:                !
   !                                                                          !
   !    http://pappulab.wustl.edu/                                            !
   !                                                                          !
   !    localCIDER is free software: you can redistribute it and/or modify    !
   !    it under the terms of the GNU General Public License as published by  !
   !    the Free Software Foundation, either version 3 of the License, or     !
   !    (at your option) any later version.                                   !
   !                                                                          !
   !    localCIDER is distributed in the hope that it will be useful,         !
   !    but WITHOUT ANY WARRANTY; without even the implied warranty of        !
   !    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         !
   !    GNU General Public License for more details.                          !
   !                                                                          !
   !    You should have received a copy of the GNU General Public License     !
   !    along with localCIDER.  If not, see <http://www.gnu.org/licenses/>.   !
   !--------------------------------------------------------------------------!
   ! AUTHORSHIP INFO:                                                         !
   !--------------------------------------------------------------------------!
   !                                                                          !
   ! MAIN AUTHOR:   Alex Holehouse                                            !
   !                                                                          !
   !--------------------------------------------------------------------------!


   Unit tests for functions from backened/sequence.py


"""
import numpy as np
import os.path
import time
import unittest
import random
from localcider.sequenceParameters import SequenceParameters
import testTools


class TestComplexityFunctions(unittest.TestCase):
    


    def setUp(self):
        self.SP_60 = SequenceParameters('QWERTYIPASDFGHKLCVNMQWERTYIPASDFGHKLCVNMQWERTYIPASDFGHKLCVNM')
        self.SP_10 = SequenceParameters('KDNIKHVPGG')

            
    def test_get_reduced_alphabet_sequence_predefined_alphabets(self):
        for i in [2,3,4,5,6,8,10,11, 12,15,18,20]:
            
            # checks that a sequence with all 20 amino acids returns a reduced-alphabet sequence 
            # made up of i-residues which contains exactly i residues!
            self.assertEqual(len(set(self.SP_60.get_reduced_alphabet_sequence(i)[0])),i)

            random_seqs = testTools.generate_random_sequence(10)

            for j in random_seqs:

                # build obj
                SP = SequenceParameters(j)

                # check reduced alphabet sequence length matches
                self.assertEqual(len(SP.get_reduced_alphabet_sequence(i)[0]), len(j))


    def test_get_linear_WF_complexity(self):
        
        # general test
        random_seqs = testTools.generate_random_sequence_list(10, minLen=15, maxLen=500)        
        for j in random_seqs:
            SequenceParameters(j).get_linear_complexity('WF',alphabetSize=5)
                    
        # test all alphabets
        for i in [2,3,4,5,6,8,10,11, 12,15,18,20]:
            for j in random_seqs:
                SequenceParameters(j).get_linear_complexity('WF',alphabetSize=i)

        # test a range of window sizes
        for i in [1,2,3,4,5,10]:
            for j in random_seqs:
                SequenceParameters(j).get_linear_complexity('WF',blobLen=i)

        # test a range of window sizes
        for i in [1,2,3,4,5,10]:
            for j in random_seqs:
                SequenceParameters(j).get_linear_complexity('WF',blobLen=i)

        # test a range of step-sizes
        for i in [1,2,3,4,5,10]:
            for j in random_seqs:
                SequenceParameters(j).get_linear_complexity('WF',stepSize=i)

        # test a range of word-sizes
        for i in [1,2,3,4,5,10]:
            for j in random_seqs:
                SequenceParameters(j).get_linear_complexity('WF',wordSize=i)

    def test_get_linear_LC_complexity(self):
        
        # general test
        random_seqs = testTools.generate_random_sequence_list(10, minLen=15, maxLen=500)        
        for j in random_seqs:
            SequenceParameters(j).get_linear_complexity('LC',alphabetSize=5)
                    
        # test all alphabets
        for i in [2,3,4,5,6,8,10,11, 12,15,18,20]:
            for j in random_seqs:
                SequenceParameters(j).get_linear_complexity('LC',alphabetSize=i)

        # test a range of window sizes
        for i in [1,2,3,4,5,10]:
            for j in random_seqs:
                SequenceParameters(j).get_linear_complexity('LC',blobLen=i)

        # test a range of window sizes
        for i in [1,2,3,4,5,10]:
            for j in random_seqs:
                SequenceParameters(j).get_linear_complexity('LC',blobLen=i)

        # test a range of step-sizes
        for i in [1,2,3,4,5,10]:
            for j in random_seqs:
                SequenceParameters(j).get_linear_complexity('LC',stepSize=i)

        # test a range of word-sizes
        for i in [1,2,3,4,5,10]:
            for j in random_seqs:
                SequenceParameters(j).get_linear_complexity('LC',wordSize=i)

    def test_get_linear_LZW_complexity(self):
        
        # general test
        random_seqs = testTools.generate_random_sequence_list(10, minLen=15, maxLen=500)        
        for j in random_seqs:
            SequenceParameters(j).get_linear_complexity('LZW',alphabetSize=5)
                    
        # test all alphabets
        for i in [2,3,4,5,6,8,10,11, 12,15,18,20]:
            for j in random_seqs:
                SequenceParameters(j).get_linear_complexity('LZW',alphabetSize=i)

        # test a range of window sizes
        for i in [1,2,3,4,5,10]:
            for j in random_seqs:
                SequenceParameters(j).get_linear_complexity('LZW',blobLen=i)

        # test a range of window sizes
        for i in [1,2,3,4,5,10]:
            for j in random_seqs:
                SequenceParameters(j).get_linear_complexity('LZW',blobLen=i)

        # test a range of step-sizes
        for i in [1,2,3,4,5,10]:
            for j in random_seqs:
                SequenceParameters(j).get_linear_complexity('LZW',stepSize=i)

        # test a range of word-sizes
        for i in [1,2,3,4,5,10]:
            for j in random_seqs:
                SequenceParameters(j).get_linear_complexity('LZW',wordSize=i)


    def test_complexity_values(self):

        out = {}

        # WF alphatbetsize 4
        out['WF'] = np.array([ 0.92321967,  0.92321967,  0.92321967,  0.88048202,  0.88048202,
                               0.92321967,  0.92321967,  0.92321967,  0.94773092,  0.92321967,
                               0.86096405,  0.86096405,  0.86096405,  0.68048202,  0.68048202,
                               0.86096405,  0.92321967,  0.92321967,  0.96096405,  0.9854753 ,
                               0.92321967,  0.92321967,  0.92321967,  0.88048202,  0.88048202,
                               0.92321967,  0.92321967,  0.92321967,  0.94773092,  0.92321967,
                               0.86096405,  0.86096405,  0.86096405,  0.68048202,  0.68048202,
                               0.86096405,  0.92321967,  0.92321967,  0.96096405,  0.9854753 ,
                               0.92321967,  0.92321967,  0.92321967,  0.88048202,  0.88048202,
                               0.92321967,  0.92321967,  0.92321967,  0.94773092,  0.92321967,
                               0.86096405])
        
        # LZW alphabate size=4
        out['LZW'] = np.array([ 0.9,  0.9,  0.9,  0.9,  0.9,  0.9,  0.9,  0.9,  0.8,  0.8,  0.8,
                                0.8,  0.8,  0.8,  0.8,  0.9,  0.9,  0.9,  1. ,  1. ,  0.9,  0.9,
                                0.9,  0.9,  0.9,  0.9,  0.9,  0.9,  0.8,  0.8,  0.8,  0.8,  0.8,
                                0.8,  0.8,  0.9,  0.9,  0.9,  1. ,  1. ,  0.9,  0.9,  0.9,  0.9,
                                0.9,  0.9,  0.9,  0.9,  0.8,  0.8,  0.8])

        # LC alphabet size = 2
        out['LC'] = np.array([ 0.625,  0.5  ,  0.5  ,  0.5  ,  0.5  ,  0.5  ,  0.625,  0.75 ,
                            0.625,  0.75 ,  0.625,  0.75 ,  0.875,  0.75 ,  0.625,  0.625,
                            0.625,  0.625,  0.75 ,  0.75 ,  0.625,  0.5  ,  0.5  ,  0.5  ,
                            0.5  ,  0.5  ,  0.625,  0.75 ,  0.625,  0.75 ,  0.625,  0.75 ,
                            0.875,  0.75 ,  0.625,  0.625,  0.625,  0.625,  0.75 ,  0.75 ,
                            0.625,  0.5  ,  0.5  ,  0.5  ,  0.5  ,  0.5  ,  0.625,  0.75 ,
                            0.625,  0.75 ,  0.625])
        
        self.assertEquals((self.SP_60.get_linear_complexity('WF',  alphabetSize=4)[1]  - out['WF'] < 0.00001).all(), True)
        self.assertEquals((self.SP_60.get_linear_complexity('LZW', alphabetSize=4)[1] - out['LZW'] < 0.00001).all(), True)
        self.assertEquals((self.SP_60.get_linear_complexity('LC',  alphabetSize=2)[1]  - out['LC'] < 0.00001).all(), True)


                                                              




"""
   !--------------------------------------------------------------------------!
   ! LICENSE INFO:                                                            !
   !--------------------------------------------------------------------------!
   !    This file is part of localCIDER.                                      !
   !                                                                          !
   !    Version 0.1.9                                                         !
   !--------------------------------------------------------------------------!

   File Description:
   ================

   This is a file containing various plotting functions. Note that localCIDER
   doesn't require these functions to work - the ability to plot things is
   nice but not essential.

   There are eight plotting functions which can be called, and are
   as follows

   >>> Show or save a single sequence on the diagam of states plot
   - show_single_phasePlot
   - save_single_phasePlot

   >>> Show or save multiple sequences on the diagram of states plot
   - show_multiple_phasePlot
   - save_multiple_phasePlot
   - show_multiple_phasePlot2
   - save_multiple_phasePlot2

   >>> Show or save a single sequence on the Uversky plot
   - show_single_uverskyPlot
   - save_single_uverskyPlot

   >>> Show or save multiple sequences on the Uversky plot
   - show_multiple_uverskyPlot
   - save_multiple_uverskyPlot
   - show_multiple_uverskyPlot2
   - save_multiple_uverskyPlot2


   These functions can be called independently of other localCIDER
   functionality, or you could calculate the fraction of positive and
   negative residues for a bunch of sequences and use these functions
   independent of the localCIDER SequenceParameter objects.

   Note that for plotting linear sequence data (e.g. linear hydropathy,
   linear NCPR etc) you must use a SequenceParameter object.

"""

from backend import plotting


#...................................................................................#
def show_single_phasePlot(
        fp,
        fn,
        label="",
        title="Diagram of states",
        legendOn=True,
        xLim=1,
        yLim=1,
        fontSize=10,
        getFig=False):
    """
    Plot a single sequence on the Pappu-Das phase plot (diagram of states).

    INPUT:
    --------------------------------------------------------------------------------
    fp         | Fraction of positive residues
    fn         | Fraction of negative residues

    label      | On-plot label of sequence (DEFAULT = no label)
    title      | Plot title (DEFAULT = "Diagram of states")
    legendOn   | Include the phase diagram region legend (DEFAULT = True)
    xLim       | Set upper limit for the x axis (DEFAULT = 1)
    yLim       | Set upper limit for the y axis (DEFAULT = 1)
    fontSize   | Set font size for the point labels (DEFAULT = 10)
    getFig     | Returns a matplotlib figure object instead of simply displaying the
               | plot on the screen (DEFAULT = False)

    OUTPUT:
    --------------------------------------------------------------------------------

    Nothing but a plot should be generated on screen
    """

    return plotting.show_single_phasePlot(
        fp, fn, label, title, legendOn, xLim, yLim, fontSize, getFig)


#...................................................................................#
def save_single_phasePlot(
        fp,
        fn,
        filename,
        label="",
        title="Diagram of states",
        legendOn=True,
        xLim=1,
        yLim=1,
        fontSize=10,
        saveFormat='png'):
    """
    Plot a single sequence on the Pappu-Das phase plot (diagram of states).

    INPUT:
    --------------------------------------------------------------------------------
    fp         | Fraction of positive residues
    fn         | Fraction of negative residues
    filename   | Path/name of file to save plot (.png is appended)

    label      | On-plot label of sequence (DEFAULT = no label)
    title      | Plot title (DEFAULT = "Diagram of states")
    legendOn   | Include the phase diagram region legend (DEFAULT = True)
    xLim       | Set upper limit for the x axis (DEFAULT = 1)
    yLim       | Set upper limit for the x axis (DEFAULT = 1)
    fontSize   | Set font size for the point labels (DEFAULT = 10)
    saveFormat | Defines the file formal to save plots as. This parameter
                 is passed to matplotlibs savefig command which supports 
                 the following filetypes: emf, eps, pdf, png, ps, raw, 
                 rgba, svg, svgz. (DEFAULT = png)

    OUTPUT:
    --------------------------------------------------------------------------------
    Nothing but a plot should be generated on screen

    """

    plotting.save_single_phasePlot(
        fp,
        fn,
        filename,
        label,
        title,
        legendOn,
        xLim,
        yLim,
        fontSize,
        saveFormat)


#...................................................................................#
def show_multiple_phasePlot(
        fp_list,
        fn_list,
        label=[""],
        title="Diagram of states",
        legendOn=True,
        xLim=1,
        yLim=1,
        fontSize=10,
        getFig=False):
    """
    Plot multiple sequences on the same Pappu-Das phase plot (diagram of states).

    INPUT:
    --------------------------------------------------------------------------------
    fp_list    | Fraction of positive residues
    fn_list    | Fraction of negative residues

    label_list | On-plot label of sequence (DEFAULT = no label)
    title      | Plot title (DEFAULT = "Diagram of states")
    legendOn   | Include the phase diagram region legend (DEFAULT = True)
    xLim       | Set upper limit for the x axis (DEFAULT = 1)
    yLim       | Set upper limit for the x axis (DEFAULT = 1)
    fontSize   | Set font size for the point labels (DEFAULT = 10)
    getFig     | Returns a matplotlib figure object instead of simply displaying the
               | plot on the screen (DEFAULT = False)


    OUTPUT:
    --------------------------------------------------------------------------------
    Nothing, but a plot with multiple points should appear on the screen

    """

    return plotting.show_multiple_phasePlot(
        fp_list,
        fn_list,
        label,
        title,
        legendOn,
        xLim,
        yLim,
        fontSize,
        getFig)


#...................................................................................#
def show_multiple_phasePlot2(
        SeqParam_list,
        label_list=[],
        title="Diagram of states",
        legendOn=True,
        xLim=1,
        yLim=1,
        fontSize=10,
        getFig=False):
    """
    Plot a single sequence on the Pappu-Das phase plot (diagram of states). This function takes
    SequenceParameter objects rather than raw values

    INPUT:
    --------------------------------------------------------------------------------
    SeqParam_list | list of sequence parameter objects

    label         | On-plot label of sequence (DEFAULT = no label)
    title         | Plot title (DEFAULT = "Diagram of states")
    legendOn      | Include the phase diagram region legend (DEFAULT = True)
    xLim          | Set upper limit for the x axis (DEFAULT = 1)
    yLim          | Set upper limit for the y axis (DEFAULT = 1)
    fontSize      | Set font size for the point labels (DEFAULT = 10)
    getFig        | Returns a matplotlib figure object instead of simply displaying the
                  | plot on the screen (DEFAULT = False)


    OUTPUT:
    --------------------------------------------------------------------------------
    Nothing, but a plot with multiple points should appear on the screen

    """

    # construct the fraction positive and negative vectors
    fp_list = []
    fn_list = []
    for seq in SeqParam_list:
        fp_list.append(seq.get_fraction_positive())
        fn_list.append(seq.get_fraction_negative())

    return plotting.show_multiple_phasePlot(
        fp_list,
        fn_list,
        label_list,
        title,
        legendOn,
        xLim,
        yLim,
        fontSize,
        getFig)


#...................................................................................#
def save_multiple_phasePlot(
        fp_list,
        fn_list,
        filename,
        label_list=[],
        title="Diagram of states",
        legendOn=True,
        xLim=1,
        yLim=1,
        fontSize=10,
        saveFormat='png'):
    """
    Plot multiple sequences on the same Pappu-Das phase plot (diagram of states) and save that file

    INPUT:
    --------------------------------------------------------------------------------
    fp_list    | Fraction of positive residues
    fn_list    | Fraction of negative residues
    filename   | name of file to save (.png is appended)

    label_list | On-plot label of sequence (DEFAULT = no label)
    title      | Plot title (DEFAULT = "Diagram of states")
    legendOn   | Include the phase diagram region legend (DEFAULT = True)
    xLim       | Set upper limit for the x axis (DEFAULT = 1)
    yLim       | Set upper limit for the y axis (DEFAULT = 1)
    fontSize   | Set font size for the point labels (DEFAULT = 10)
    saveFormat | Defines the file formal to save plots as. This parameter
                 is passed to matplotlibs savefig command which supports 
                 the following filetypes: emf, eps, pdf, png, ps, raw, 
                 rgba, svg, svgz. (DEFAULT = png)



    OUTPUT:
    --------------------------------------------------------------------------------
    No output, but if succesful a file with filename is generated with the associated plot

    """

    plotting.save_multiple_phasePlot(
        fp_list,
        fn_list,
        filename,
        label_list,
        title,
        legendOn,
        xLim,
        yLim,
        fontSize,
        saveFormat)


#...................................................................................#
def save_multiple_phasePlot2(
        SeqParam_list,
        filename,
        label_list=[],
        title="Diagram of states",
        legendOn=True,
        xLim=1,
        yLim=1,
        fontSize=10,
        saveFormat='png'):
    """
    Plot multiple sequences on the same Pappu-Das phase plot (diagram of states) and save that file.
    This function takes SequenceParameter objects rather than raw values.

    INPUT:
    --------------------------------------------------------------------------------
    SeqParam_list | list of sequence parameter objects
    filename      | name of file to save (.png is appended)

    label_list    | On-plot label of sequence (DEFAULT = no label)
    title         | Plot title (DEFAULT = "Diagram of states")
    legendOn      | Include the phase diagram region legend (DEFAULT = True)
    xLim          | Set upper limit for the x axis (DEFAULT = 1)
    yLim          | Set upper limit for the y axis (DEFAULT = 1)
    fontSize      | Set font size for the point labels (DEFAULT = 10)
    saveFormat    | Defines the file formal to save plots as. This parameter
                    is passed to matplotlibs savefig command which supports 
                    the following filetypes: emf, eps, pdf, png, ps, raw, 
                    rgba, svg, svgz. (DEFAULT = png)

    OUTPUT:
    --------------------------------------------------------------------------------
    No output, but if succesful a file with filename is generated with the associated plot

    """

    # construct the fraction positive and negative vectors
    fp_list = []
    fn_list = []
    for seq in SeqParam_list:
        fp_list.append(seq.get_fraction_positive())
        fn_list.append(seq.get_fraction_negative())

    plotting.save_multiple_phasePlot(
        fp_list,
        fn_list,
        filename,
        label_list,
        title,
        legendOn,
        xLim,
        yLim,
        fontSize,
        saveFormat)


####################################################
##
# UVERSKY PLOTS BELOW
##
####################################################

#...................................................................................#
def show_single_uverskyPlot(
        hydropathy,
        mean_net_charge,
        label="",
        title="Uversky plot",
        legendOn=True,
        xLim=1,
        yLim=1,
        fontSize=10,
        getFig=False):
    """
    Plots a single sequence on the Uversky plot (hydropathy vs. mean net charge)

    INPUT:
    --------------------------------------------------------------------------------
    hydropathy      | Mean hydropathy for sequence
    mean_net_charge | Absolute magnitude of the protein's net charge divided by sequence length

    label           | On-plot label of sequence (DEFAULT = no label)
    title           | Plot title (DEFAULT = "Uversky plot")
    legendOn        | Include the phase diagram region legend (DEFAULT = True)
    xLim            | Set upper limit for the x axis (DEFAULT = 1)
    yLim            | Set upper limit for the y axis (DEFAULT = 1)
    fontSize        | Set font size for the point labels (DEFAULT = 10)
    getFig          | Returns a matplotlib figure object instead of simply displaying the
                    | plot on the screen (DEFAULT = False)



    OUTPUT:
    --------------------------------------------------------------------------------
    Nothing, but an annotated Uversky plot should be generated on screen

    """

    return plotting.show_single_uverskyPlot(
        hydropathy,
        mean_net_charge,
        label,
        title,
        legendOn,
        xLim,
        yLim,
        fontSize,
        getFig)


#...................................................................................#
def save_single_uverskyPlot(
        hydropathy,
        mean_net_charge,
        filename,
        label="",
        title="Uversky plot",
        legendOn=True,
        xLim=1,
        yLim=1,
        fontSize=10,
        saveFormat='png'):
    """
    Plots a single sequence on the Uversky plot (hydropathy vs. mean net charge) and save it to 'filename' (.png is
    appended).

    INPUT:
    --------------------------------------------------------------------------------
    hydropathy      | Mean hydropathy for sequence
    mean_net_charge | Absolute magnitude of the protein's net charge divided by sequence length
    filename        | Path/name of file to save plot (.png is appended)

    label           | On-plot label of sequence (DEFAULT = no label)
    title           | Plot title (DEFAULT = "Uversky plot")
    legendOn        | Include the phase diagram region legend (DEFAULT = True)
    xLim            | Set upper limit for the x axis (DEFAULT = 1)
    yLim            | Set upper limit for the y axis (DEFAULT = 1)
    fontSize        | Set font size for the point labels (DEFAULT = 10)
    saveFormat      | Defines the file formal to save plots as. This parameter
                      is passed to matplotlibs savefig command which supports 
                      the following filetypes: emf, eps, pdf, png, ps, raw, 
                      rgba, svg, svgz. (DEFAULT = png)


    OUTPUT:
    --------------------------------------------------------------------------------
    Nothing but single uversky plot should be saved to disk

    """

    plotting.save_single_uverskyPlot(
        hydropathy,
        mean_net_charge,
        filename,
        label,
        title,
        legendOn,
        xLim,
        yLim,
        fontSize,
        saveFormat)


#...................................................................................#
def show_multiple_uverskyPlot(
        hydropathy_list,
        mean_net_charge_list,
        label_list=[],
        title="Uversky plot",
        legendOn=True,
        xLim=1,
        yLim=1,
        fontSize=10,
        getFig=False):
    """
    Plots multiple sequences on the Uversky plot (hydropathy vs. mean net charge) and shows the plot on the screen.

    INPUT:
    --------------------------------------------------------------------------------
    hydropathy_list      | List of proteins' mean hydropathy
    mean_net_charge_list | List of the absolute magnitude of the protein's net charge divided by sequence length

    label_list           | List of labels for each sequence (empty = no list)
    title                | Plot title (DEFAULT = "Uversky plot")
    legendOn             | Include the phase diagram region legend (DEFAULT = True)
    xLim                 | Set upper limit for the x axis (DEFAULT = 1)
    yLim                 | Set upper limit for the y axis (DEFAULT = 1)
    fontSize             | Set font size for the point labels (DEFAULT = 10)
    getFig               | Returns a matplotlib figure object instead of simply displaying the
                         | plot on the screen (DEFAULT = False)


    OUTPUT:
    --------------------------------------------------------------------------------
    Nothing, but a Uversky plot with multiple points should appear on the screen

    """
    return plotting.show_multiple_uverskyPlot(
        hydropathy_list,
        mean_net_charge_list,
        label_list,
        title,
        legendOn,
        xLim,
        yLim,
        fontSize,
        getFig)


#...................................................................................#
def show_multiple_uverskyPlot2(
        SeqParam_list,
        label_list=[],
        title="Uversky plot",
        legendOn=True,
        xLim=1,
        yLim=1,
        fontSize=10,
        getFig=False):
    """
    Plots multiple sequences on the Uversky plot (hydropathy vs. mean net charge) and shows the plot on the screen.
    This function takes a list of SequenceParameter objects instead of the actual values

    INPUT:
    --------------------------------------------------------------------------------
    SeqParam_list        | List of sequence parameter objects

    label_list           | List of labels for each sequence (empty = no list)
    title                | Plot title (DEFAULT = "Uversky plot")
    legendOn             | Include the phase diagram region legend (DEFAULT = True)
    xLim                 | Set upper limit for the x axis (DEFAULT = 1)
    yLim                 | Set upper limit for the y axis (DEFAULT = 1)
    fontSize             | Set font size for the point labels (DEFAULT = 10)
    getFig               | Returns a matplotlib figure object instead of simply displaying the
                         | plot on the screen (DEFAULT = False)


    OUTPUT:
    --------------------------------------------------------------------------------
    Nothing, but a Uversky plot with multiple points should appear on the screen

    """

    # construct the fraction positive and negative vectors
    hydropathy_list = []
    mean_net_charge_list = []
    for seq in SeqParam_list:

        hydropathy_list.append(seq.get_uversky_hydropathy())
        mean_net_charge_list.append(seq.get_mean_net_charge())

    return plotting.show_multiple_uverskyPlot(
        hydropathy_list,
        mean_net_charge_list,
        label_list,
        title,
        legendOn,
        xLim,
        yLim,
        fontSize,
        getFig)


#...................................................................................#
def save_multiple_uverskyPlot(
        hydropathy_list,
        mean_net_charge_list,
        filename,
        label_list=[],
        title="Uversky plot",
        legendOn=True,
        xLim=1,
        yLim=1,
        fontSize=10,
        saveFormat='png'):
    """
    Plots multiple sequences on the Uversky plot (hydropathy vs. mean net charge) and saves that plot to 'filename' (.png
    is appended).

    INPUT:
    --------------------------------------------------------------------------------
    hydropathy           | List of mean hydropathies for sequences
    mean_net_charge_list | List of the absolute magnitude of the protein's net charge divided by sequence length
    filename             | Path/name of file to save plot (.png is appended)

    label_list           | List of labels for each sequence (empty = no list)
    title                | Plot title (DEFAULT = "Uversky plot")
    legendOn             | Include the phase diagram region legend (DEFAULT = True)
    xLim                 | Set upper limit for the x axis (DEFAULT = 1)
    yLim                 | Set upper limit for the y axis (DEFAULT = 1)
    fontSize             | Set font size for the point labels (DEFAULT = 10)
    saveFormat           | Defines the file formal to save plots as. This parameter
                           is passed to matplotlibs savefig command which supports 
                           the following filetypes: emf, eps, pdf, png, ps, raw, 
                           rgba, svg, svgz. (DEFAULT = png)



    OUTPUT:
    --------------------------------------------------------------------------------
    Nothing, but a Uversky plot with multiple points should be saved to the 'filename', where filename
    defines the name of a file/path for saving

    """

    plotting.save_multiple_uverskyPlot(
        hydropathy_list,
        mean_net_charge_list,
        filename,
        label_list,
        title,
        legendOn,
        xLim,
        yLim,
        fontSize,
        saveFormat)


#...................................................................................#
def save_multiple_uverskyPlot2(
        SeqParam_list,
        filename,
        label_list=[],
        title="Uversky plot",
        legendOn=True,
        xLim=1,
        yLim=1,
        fontSize=10,
        saveFormat='png'):
    """
    Plots multiple sequences on the Uversky plot (hydropathy vs. mean net charge) and saves that plot to 'filename' (.png
    is appended). This function takes SequenceParameter objects instead of list of hydropathy and mean_net_charge
    values

    INPUT:
    --------------------------------------------------------------------------------
    SeqParam_list   | List of sequence parameter objects
    filename        | Path/name of file to save plot (.png is appended)

    label           | On-plot label of sequence (DEFAULT = no label)
    title           | Plot title (DEFAULT = "Uversky plot")
    legendOn        | Include the phase diagram region legend (DEFAULT = True)
    xLim            | Set upper limit for the x axis (DEFAULT = 1)
    yLim            | Set upper limit for the y axis (DEFAULT = 1)
    fontSize        | Set font size for the point labels (DEFAULT = 10)
    saveFormat      | Defines the file formal to save plots as. This parameter
                      is passed to matplotlibs savefig command which supports 
                      the following filetypes: emf, eps, pdf, png, ps, raw, 
                      rgba, svg, svgz. (DEFAULT = png)


    OUTPUT:
    --------------------------------------------------------------------------------
    Nothing, but a Uversky plot with multiple points should be saved to the 'filename', where filename
    defines the name of a file/path for saving

    """

    # construct the fraction positive and negative vectors
    hydropathy_list = []
    mean_net_charge_list = []
    for seq in SeqParam_list:
        hydropathy_list.append(seq.get_uversky_hydropathy())
        mean_net_charge_list.append(seq.get_mean_net_charge())

    plotting.save_multiple_uverskyPlot(
        hydropathy_list,
        mean_net_charge_list,
        filename,
        label_list,
        title,
        legendOn,
        xLim,
        yLim,
        fontSize)



"""
   !--------------------------------------------------------------------------!
   ! LICENSE INFO:                                                            !
   !--------------------------------------------------------------------------!
   !    This file is part of localCIDER.                                      !
   !                                                                          !
   !    Version 0.1.9                                                         !
   !                                                                          !
   !    Copyright (C) 2014 - 2016                                             !
   !    The localCIDER development team (current and former contributors)     !
   !    Alex Holehouse, James Ahad, Rahul K. Das.                             !
   !                                                                          !
   !    localCIDER was developed in the lab of Rohit Pappu at Washington      !
   !    University in St. Louis. Please see the website for citation          !
   !    information:                                                          !
   !                                                                          !
   !    http://pappulab.github.io/localCIDER/                                 !
   !                                                                          !
   !    For more information please see the Pappu lab website:                !
   !                                                                          !
   !    http://pappulab.wustl.edu/                                            !
   !                                                                          !
   !    localCIDER is free software: you can redistribute it and/or modify    !
   !    it under the terms of the GNU General Public License as published by  !
   !    the Free Software Foundation, either version 3 of the License, or     !
   !    (at your option) any later version.                                   !
   !                                                                          !
   !    localCIDER is distributed in the hope that it will be useful,         !
   !    but WITHOUT ANY WARRANTY; without even the implied warranty of        !
   !    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         !
   !    GNU General Public License for more details.                          !
   !                                                                          !
   !    You should have received a copy of the GNU General Public License     !
   !    along with localCIDER.  If not, see <http://www.gnu.org/licenses/>.   !
   !--------------------------------------------------------------------------!
   ! AUTHORSHIP INFO:                                                         !
   !--------------------------------------------------------------------------!
   !                                                                          !
   ! MAIN AUTHOR:   Alex Holehouse                                            !
   !                                                                          !
   !--------------------------------------------------------------------------!

   Unit tests for SequenceParameter functions

"""

import os.path
import time
import unittest
import random

import numpy as np

from localcider import sequenceParameters, plots
import testTools


class TestSequenceParametersFunctions(unittest.TestCase):

    def setUp(self):
        self.testObj = sequenceParameters.SequenceParameters(
            "MDVFMKGLSKAKEGVVAAAEKTKQGVAEAAGKTKEGVLYVGSKTKEGVVHGVATVAEKTKEQVTNVGGAVVTGVTAVAQKTVEGAGSIAAATGFVKKDQLGKNEEGAPQEGILEDMPVDPDNEAYEMPSEEGYQDYEPEA")

    # <><><><><><><><><><><><><><><><><><><><><><><><><><><><>
    #
    # SECTION 1: Testing SequenceParameter object functions
    #
    # <><><><><><><><><><><><><><><><><><><><><><><><><><><><>

    def test_clear_phosphosites(self):
        # set phosphosites
        # self.testObj.
        pass

    def test_get_sequence(self):
        seq = self.testObj.get_sequence()
        self.assertEqual(
            seq,
            "MDVFMKGLSKAKEGVVAAAEKTKQGVAEAAGKTKEGVLYVGSKTKEGVVHGVATVAEKTKEQVTNVGGAVVTGVTAVAQKTVEGAGSIAAATGFVKKDQLGKNEEGAPQEGILEDMPVDPDNEAYEMPSEEGYQDYEPEA")

    def test_get_mean_hydropathy(self):
        MH = self.testObj.get_mean_hydropathy()
        self.assertEqual(MH, 4.097142857142858)

    def test_get_uversky_hydropathy(self):
        MUH = self.testObj.get_uversky_hydropathy()
        self.assertEqual(MUH, 0.4552380952380952)

    def test_get_fraction_disorder_promoting(self):
        FD = self.testObj.get_fraction_disorder_promoting()
        self.assertEqual(FD, 0.7285714285714285)

    def test_get_amino_acid_fractions(self):
        AADict = self.testObj.get_amino_acid_fractions()

        PC = {'A': 0.1357142857142857,
              'C': 0.0,
              'D': 0.04285714285714286,
              'E': 0.12857142857142856,
              'F': 0.014285714285714285,
              'G': 0.12857142857142856,
              'H': 0.007142857142857143,
              'I': 0.014285714285714285,
              'K': 0.10714285714285714,
              'L': 0.02857142857142857,
              'M': 0.02857142857142857,
              'N': 0.02142857142857143,
              'P': 0.03571428571428571,
              'Q': 0.04285714285714286,
              'R': 0.0,
              'S': 0.02857142857142857,
              'T': 0.07142857142857142,
              'V': 0.1357142857142857,
              'W': 0.0,
              'Y': 0.02857142857142857}

        for AA in AADict:
            self.assertEqual(AADict[AA], PC[AA])

    def test_get_kappa(self):
        KP = self.testObj.get_kappa()
        self.assertEqual(KP, 0.17167490749129125)

    def test_get_countPos(self):
        CP = self.testObj.get_countPos()
        self.assertEqual(CP, 15)

    def test_get_countNeg(self):
        CN = self.testObj.get_countNeg()
        self.assertEqual(CN, 24)

    def test_get_countNeut(self):
        CNeu = self.testObj.get_countNeut()
        self.assertEqual(CNeu, 101)

    def test_get_fraction_positive(self):
        FPos = self.testObj.get_fraction_positive()
        self.assertEqual(FPos, 0.10714285714285714)

    def test_get_fraction_negative(self):
        FNeg = self.testObj.get_fraction_negative()
        self.assertEqual(FNeg, 0.17142857142857143)

    def test_get_FCR(self):
        FCR = self.testObj.get_FCR()
        self.assertEqual(FCR, 0.2785714285714286)

    def test_get_NCPR(self):
        NCPR = self.testObj.get_NCPR()
        self.assertEqual(NCPR, -0.06428571428571428)

    def test_get_mean_net_charge(self):
        MNC = self.testObj.get_mean_net_charge()
        self.assertEqual(MNC, 0.06428571428571428)

    def test_get_phasePlotRegion(self):
        PPR = self.testObj.get_phasePlotRegion()
        self.assertEqual(PPR, 2)

    def test_get_phosphosites(self):
        PS_pre = self.testObj.get_phosphosites()
        self.assertEqual(len(PS_pre), 0)

    def test_get_kappa_after_phosphorylation(self):

        # check we're starting with the phosphosites set empty
        self.assertEqual(len(self.testObj.get_phosphosites()), 0)

        # run and test phosphorylation
        self.testObj.set_phosphosites([33, 39, 42])
        self.assertEqual(len(self.testObj.get_phosphosites()), 3)

        PostPhosK = self.testObj.get_kappa_after_phosphorylation()
        self.assertEqual(PostPhosK, 0.15101360943849682)

        pass

    def test_get_all_phosphorylatable_sites(self):

        all_sites = self.testObj.get_all_phosphorylatable_sites()

        self.assertEqual(
            all_sites, [
                9, 22, 33, 39, 42, 44, 54, 59, 64, 72, 75, 81, 87, 92, 125, 129, 133, 136])

    def test_get_full_phosphostatus_kappa_distribution(self):

        # just to be sure
        self.testObj.clear_phosphosites()
        self.testObj.set_phosphosites([33, 39, 42])

        FPKD = [(0.17167490749129125,
                 0.10714285714285714,
                 0.17142857142857143,
                 0.2785714285714286,
                 -0.06428571428571428,
                 4.097142857142858,
                 ('0', '0', '0')),
                (0.16068627291329185,
                 0.10714285714285714,
                 0.17857142857142858,
                 0.2857142857142857,
                 -0.07142857142857142,
                 4.077857142857144,
                 ('0', '0', '1')),
                (0.16863038053131135,
                 0.10714285714285714,
                 0.17857142857142858,
                 0.2857142857142857,
                 -0.07142857142857142,
                 4.081428571428572,
                 ('0', '1', '0')),
                (0.1609064850879989,
                 0.10714285714285714,
                 0.18571428571428572,
                 0.29285714285714287,
                 -0.07857142857142857,
                 4.062142857142857,
                 ('0', '1', '1')),
                (0.161014373883184,
                 0.10714285714285714,
                 0.17857142857142858,
                 0.2857142857142857,
                 -0.07142857142857142,
                 4.077142857142858,
                 ('1', '0', '0')),
                (0.15064319064948062,
                 0.10714285714285714,
                 0.18571428571428572,
                 0.29285714285714287,
                 -0.07857142857142857,
                 4.057857142857142,
                 ('1', '0', '1')),
                (0.15825014591368247,
                 0.10714285714285714,
                 0.18571428571428572,
                 0.29285714285714287,
                 -0.07857142857142857,
                 4.061428571428571,
                 ('1', '1', '0')),
                (0.15101360943849682,
                 0.10714285714285714,
                 0.19285714285714287,
                 0.3,
                 -0.08571428571428572,
                 4.042142857142857,
                 ('1', '1', '1'))]

        FPKD_calc = self.testObj.get_full_phosphostatus_kappa_distribution()

        self.assertEqual(FPKD_calc, FPKD)

    def test_set_phosphosites(self):

        # note we also check that incorrect phosphosites are not
        # assigned

        self.testObj.clear_phosphosites()

        print ""
        print "###### WE EXPECT A WARNING ON THE LINE BELOW ######"
        self.testObj.set_phosphosites([33, 39, 42, 43])
        print "###################################################"
        self.assertEqual(self.testObj.get_phosphosites(), [33, 39, 42])

    def test_save_phaseDiagramPlot(self):
        # Should test the creation date but that's for a later update...
        # note that if we can create and save the file we can also display it
        self.testObj.save_phaseDiagramPlot("tmpfiles/phaseplot_test")

    def test_save_uverskyPlot(self):
        #
        #
        self.testObj.save_uverskyPlot("tmpfiles/uversky_test")

    def test_general_coverage(self):

        AAs = list("QWERTYIPASDFGHKLCVNM")

        # generate 5 random sequences
        # LENGTH BETWEEN 1 AND 150
        random.seed()

        seq_list = []
        for i in xrange(0, 10):
            L1 = random.randrange(200, 1000)

            S1 = ""
            for i in xrange(0, L1):
                S1 = S1 + random.choice(AAs)

            seq_list.append(S1)

        pos = 0
        print ""
        
        # generate a list of 10 random sequences between
        # length 100 and 1000
        seq_list = testTools.generate_random_sequence_list(number=10, minLen=100, maxLen=1000)
            

        # some edge cases...
        seq_list.extend(testTools.generate_random_sequence_list(number=1, minLen=3,  maxLen=3))
        seq_list.extend(testTools.generate_random_sequence_list(number=1, minLen=4,  maxLen=4))
        seq_list.extend(testTools.generate_random_sequence_list(number=1, minLen=5,  maxLen=5))
        seq_list.extend(testTools.generate_random_sequence_list(number=1, minLen=6,  maxLen=6))
        seq_list.extend(testTools.generate_random_sequence_list(number=1, minLen=7,  maxLen=7))
        seq_list.extend(testTools.generate_random_sequence_list(number=1, minLen=8,  maxLen=8))
        seq_list.extend(testTools.generate_random_sequence_list(number=1, minLen=9,  maxLen=9))
        seq_list.extend(testTools.generate_random_sequence_list(number=1, minLen=10, maxLen=10))
        seq_list.extend(testTools.generate_random_sequence_list(number=1, minLen=11, maxLen=11))
        seq_list.extend(testTools.generate_random_sequence_list(number=1, minLen=12, maxLen=12))

        seq_list.append('EEEEEEEEEE')
        seq_list.append('KKKKKKKKKK')
        seq_list.append('EKEKEKEKEK')
        seq_list.append('GGGGGGGGGG')

        seq_list.append('EEEE')
        seq_list.append('KKKK')
        seq_list.append('EKEK')
        seq_list.append('GGGG')

        seq_list.append('EEEEE')
        seq_list.append('KKKKK')
        seq_list.append('EKEKE')
        seq_list.append('GGGGG')
            


        for i in seq_list:
            print ""
            print "Sequence " + str(pos)
            print i
            iSEQ = sequenceParameters.SequenceParameters(i)

            iSEQ.get_FCR()
            iSEQ.get_NCPR()
            iSEQ.get_countNeg()
            iSEQ.get_countPos()
            iSEQ.get_fraction_positive()
            iSEQ.get_fraction_negative()
            iSEQ.get_fraction_disorder_promoting()
            iSEQ.get_amino_acid_fractions()
            iSEQ.get_kappa()
            iSEQ.get_mean_net_charge()
            iSEQ.get_phasePlotRegion()
            iSEQ.get_mean_hydropathy()
            iSEQ.get_uversky_hydropathy()
            iSEQ.get_HTMLColorString()
            iSEQ.get_kappa_proline()
            iSEQ.get_PPII_propensity()
            iSEQ.get_kappa_proline_sequence()
            #iSEQ.
            iSEQ.get_delta()
            iSEQ.get_deltaMax()
            
            # check various reduced alphabets
            for ABS in [2,3, 4,5,6,8,10,11,12,15,18,20]:
                iSEQ.get_reduced_alphabet_sequence(ABS)


            # actually check this...
            self.assertEqual(iSEQ.get_length(), len(iSEQ))

            # get linear vectors...
            max_blobsize=len(i)
            iSEQ.get_linear_FCR(min(5,max_blobsize))
            iSEQ.get_linear_FCR(1)
            iSEQ.get_linear_FCR(min(10, max_blobsize))

            iSEQ.get_linear_NCPR(min(5,max_blobsize))
            iSEQ.get_linear_NCPR(1)
            iSEQ.get_linear_NCPR(min(10, max_blobsize))

            iSEQ.get_linear_hydropathy(min(5,max_blobsize))
            iSEQ.get_linear_hydropathy(1)
            iSEQ.get_linear_hydropathy(min(10, max_blobsize))

            # get complexity vectors (simple checks, more in depth in the
            # complexity test file...)
            for CT in ['WF','LC','LZW']:
                for ABS in [2,5,10,20]:
                    for windowsize in [1,2,3]:
                        iSEQ.get_linear_complexity(complexityType=CT, alphabetSize=ABS, blobLen=windowsize)
                        

            
            psites = iSEQ.get_all_phosphorylatable_sites()
            print "psites"
            print psites
            if len(psites) > 0:

                # grab 3 sites randomly
                sites = []
                for i in xrange(0, 2):
                    sites.append(random.choice(psites))
                iSEQ.set_phosphosites(sites)
                iSEQ.get_kappa_after_phosphorylation()
                iSEQ.get_phosphosequence()
                iSEQ.get_full_phosphostatus_kappa_distribution()
                iSEQ.clear_phosphosites()

            # single
            iSEQ.save_phaseDiagramPlot("tmpfiles/phase_test_S" + str(pos))
            iSEQ.save_uverskyPlot("tmpfiles/uversky_test_S" + str(pos))

            # try with a blobval
            iSEQ.save_linearNCPR("tmpfiles/NCPR_test_S" + str(pos), min(5, max_blobsize))
            iSEQ.save_linearFCR("tmpfiles/FCR_test_S" + str(pos), min(5, max_blobsize))
            iSEQ.save_linearHydropathy(
                "tmpfiles/Hydropathy_test_S" + str(pos), min(5, max_blobsize))
            iSEQ.save_linearSigma("tmpfiles/sigma_test_S" + str(pos), min(5, max_blobsize))

            # try with default
            iSEQ.save_linearNCPR("tmpfiles/NCPR_test_S" + str(pos), min(5, max_blobsize))
            iSEQ.save_linearFCR("tmpfiles/FCR_test_S" + str(pos), min(5, max_blobsize))
            iSEQ.save_linearHydropathy("tmpfiles/Hydropathy_test_S" + str(pos), min(5, max_blobsize))
            iSEQ.save_linearSigma("tmpfiles/sigma_test_S" + str(pos), min(5, max_blobsize))

            pos = pos + 1

    def test_pathological_sequences(self):

        # first our ability to reproduce the Das paper sequences
        das = [
            'EKEKEKEKEKEKEKEKEKEKEKEKEKEKEKEKEKEKEKEKEKEKEKEKEK',
            'EEEKKKEEEKKKEEEKKKEEEKKKEEEKKKEEEKKKEEEKKKEEEKKKEK',
            'KEKKKEKKEEKKEEKEKEKEKEEKKKEEKEKEKEKKKEEKEKEEKKEEEE',
            'KEKEEKEKKKEEEEKEKKKKEEKEKEKEKEEKKEEKKKKEEKEEKEKEKE',
            'KEKEKKEEKEKKEEEKKEKEKEKKKEEKKKEEKEEKKEEKKKEEKEEEKE',
            'EEEKKEKKEEKEEKKEKKEKEEEKKKEKEEKKEEEKKKEKEEEEKKKKEK',
            'EEEEKKKKEEEEKKKKEEEEKKKKEEEEKKKKEEEEKKKKEEEEKKKKEK',
            'KKKKEEEEKKKKEEEEKKKKEEEEKKKKEEEEKKKKEEEEKKKKEEEEKE',
            'EEKKEEEKEKEKEEEEEKKEKKEKKEKKKEEKEKEKKKEKKKKEKEEEKE',
            'EKKKKKKEEKKKEEEEEKKKEEEKKKEKKEEKEKEEKEKKEKKEEKEEEE',
            'EKEKKKKKEEEKKEKEEEEKEEEEKKKKKEKEEEKEEKKEEKEKKKEEKK',
            'EKKEEEEEEKEKKEEEEKEKEKKEKEEKEKKEKKKEKKEEEKEKKKKEKK',
            'KEKKKEKEKKEKKKEEEKKKEEEKEKKKEEKKEKKEKKEEEEEEEKEEKE',
            'EKKEKEEKEEEEKKKKKEEKEKKEKKKKEKKKKKEEEEEEKEEKEKEKEE',
            'KKEKKEKKKEKKEKKEEEKEKEKKEKKKKEKEKKEEEEEEEEKEEKKEEE',
            'EKEKEEKKKEEKKKKEKKEKEEKKEKEKEKKEEEEEEEEEKEKKEKKKKE',
            'EKEKKKKKKEKEKKKKEKEKKEKKEKEEEKEEKEKEKKEEKKEEEEEEEE',
            'KEEKKEEEEEEEKEEKKKKKEKKKEKKEEEKKKEEKKKEEEEEEKKKKEK',
            'EEEEEKKKKKEEEEEKKKKKEEEEEKKKKKEEEEEKKKKKEEEEEKKKKK',
            'EEKEEEEEEKEEEKEEKKEEEKEKKEKKEKEEKKEKKKKKKKKKKKKEEE',
            'EEEEEEEEEKEKKKKKEKEEKKKKKKEKKEKKKKEKKEEEEEEKEEEKKK',
            'KEEEEKEEKEEKKKKEKEEKEKKKKKKKKKKKKEKKEEEEEEEEKEKEEE',
            'EEEEEKEEEEEEEEEEEKEEKEKKKKKKEKKKKKKKEKEKKKKEKKEEKK',
            'EEEEKEEEEEKEEEEEEEEEEEEKKKEEKKKKKEKKKKKKKEKKKKKKKK',
            'EEEEEEEEEEEKEEEEKEEKEEKEKKKKKKKKKKKKKKKKKKEEKKEEKE',
            'KEEEEEEEKEEKEEEEEEEEEKEEEEKEEKKKKKKKKKKKKKKKKKKKKE',
            'KKEKKKEKKEEEEEEEEEEEEEEEEEEEEKEEKKKKKKKKKKKKKKKEKK',
            'EKKKKKKKKKKKKKKKKKKKKKEEEEEEEEEEEEEEEEEEKKEEEEEKEK',
            'KEEEEKEEEEEEEEEEEEEEEEEEEEEKKKKKKKKKKKKKKKKKKKKKKK',
            'EEEEEEEEEEEEEEEEEEEEEEEEEKKKKKKKKKKKKKKKKKKKKKKKKK']

        das_kappa = [
            0.000880589547008,
            0.00256304234944,
            0.0138226351796,
            0.024499790002,
            0.0139736279003,
            0.0272607247241,
            0.0450342748224,
            0.0450342748224,
            0.0624156640006,
            0.083437842179,
            0.0840461196368,
            0.0864189374177,
            0.0951073474213,
            0.131060316051,
            0.135374506326,
            0.145853296103,
            0.16433443748,
            0.167720875959,
            0.194104529923,
            0.27210316789,
            0.273742292347,
            0.321856766152,
            0.354595821897,
            0.445561294563,
            0.528308824282,
            0.610154441657,
            0.672910640197,
            0.766654273884,
            0.876364376789,
            1.0]

        count = 0
        for i in das:
            a = sequenceParameters.SequenceParameters(i)

            self.assertEqual(
                round(
                    das_kappa[count], 12), round(
                    a.get_kappa(), 12))
            count = count + 1

        Pathoseqs = [
            'EEEEEEEEEEEDDEEEEGDD',
            'EEEDDEEEEDESSSSGGGEE',
            'EEEEEFGLDEEDEDEDEDEE',
            'EEEEEDEDEDEDEDEAGSEL',
            'EEEDDEDDEDSEEDSEDDED',
            'EEDAECIDDDEEDEEDEEED',
            'EDDEEAEEEEEEEEEEEDED',
            'DGEDEDDEDDDDDDDDDDDD',
            'SEEEEEEKEEEEEEEEEEEE',
            'EEKEEEEEEEEEEEEEEEDE',
            'EDSNEDEEEDDEEEDEEDDE',
            'DEEEDDEEEDEEDDEDDESD',
            'DDDDDDDSDDQGDEDDEDEE',
            'EEDDEEEEEEEEEEEEELTE',
            'EEDEEDEEEEESESSDSEEE',
            'EGADMEEEEEEEEEEEEEEE',
            'DDDDEDDDEDDDDEDLRTDS',
            'EEEEEEEEEEEEETGSNSEE',
            'SDSDSEEEDDEEEDDEDEDD',
            'EGLGVQGAEEEEEEEEEEEE',
            'ESEEGQEDEDEEDEEDEDEE',
            'DEDEEEEDEEEDEEDKDADS',
            'EEEKEDEEEEEEEEEEEEEE',
            'DEEEEEEEEEEEEEEEVTEV',
            'EPIEEEEEEEEEEEEEEEED',
            'EEEEEEEEEEEEEEDQDMDA',
            'ESDEEEEEEEEEEEEEDDDD',
            'EEEEEEEEEEEDDDDDKGDG',
            'DDEDDEESDEEEEEEEEEEE',
            'EEEEEEEEEEEEATDSEEEE',
            'QEEGGEEEEEEEEEEEEEEE',
            'EEEEEDEEEEEEEDSIVDDA',
            'EEEEDEEEEGEEGEEDEEDE',
            'EQLSEEEEEEEEEEEEEEEE',
            'EEEEEEEEEEEAEEEEEEED',
            'DEEEDEEEEEEDEEALLEDE',
            'EEEEEEEELPEDDEEEEEEE',
            'EEELPEDDEEEEEEEEEDDD',
            'EEEDDEDEEEEEEEEEGDGE',
            'ESSEEEEEEEDEEEEEEEEE',
            'EEEEEEEEEEEEEEEEEGEE',
            'EENDDQEEEEEDEDDEDDEE',
            'EDEEEEDDDDDDEGEDDGEE',
            'PLDKACAEDDDEEDEEEEEE',
            'DDEDDDEAEDNDEDEDDDEE',
            'DEDDDEDGEDVEDEEEEEEE',
            'EDGEDVEDEEEEEEEEEEEE',
            'EEEEEEEEEEEEEEEEAAPD',
            'EEEEEEEEQEEEEEEEEEEE',
            'EEDEDEDLEEEEEEEEEDDD',
            'EEEEGGGQGEEEEEEEEDEE',
            'EEEEEEEDDEDEDADISLEE',
            'DDDDDDEEDDDEDDDDDDFD',
            'DEEEDDDSEEDEEDDEDEDE',
            'EEEEEEEEEEEEEDFEEEEE',
            'EEEEEDEEEYFEEEEEEEEE',
            'EEEEGELEEEEEEEDEEEEE',
            'DSSSSSEDEEEEEEEEEDED',
            'ELGYPREEDEEEEEDDEEEE',
            'DLGEEEEEEEEEDEEEEEDD',
            'EEDEEEEEDDDDDELEDEGE',
            'EDEEEEEEEEEEEKEEEEEW',
            'EEAEEEEEEDEEEEEEEEEE',
            'DDDDDDDEEDGVFDDEDEEE',
            'EEEEEEEEEAPVSVWDEEED',
            'EDDDDDDDEDDDDEEENAED',
            'EDEEEEEEEEEEDEDEDLEE',
            'EEEDDDEDEDEEDDVSEGSE',
            'EEEEEEEEEEEEEEEAEEEE',
            'DEEEEEEEEEYDEEEEEEDD',
            'DEDDEDEDEDEDEDEDEDKE',
            'DEDEDEDEDEDEDKEEEEED',
            'DASDNEEEEEEEEEEEEEEE',
            'EEVDQQEEEEEEEEEEEEEE',
            'EEEEEEEEEEEAAAAVALGE',
            'EDEDDEDEDEEEEDDENGDS',
            'ENEEDDEDEDDDEDDDEDED',
            'DEDEDDDEDDDEDEDNESEG',
            'EVDEDGEEEEEEEEEEEEEE',
            'EEEEEEEEEEEEEEEEYEQD',
            'EEEEEEDGHSEQEEEEEEEE',
            'EEDDKEDDDDDEDDDDEEDE',
            'DEEDEEEEEEEEEDDDDDTE',
            'EMEESEEDEEEEDEEEEEED',
            'EEDEEEEEEDEEESKAGGED',
            'EEEEEAEEEEEEEEEEEEEE',
            'EERNGLEEEEEDDEEDEEDD']

        for i in Pathoseqs:
            a = sequenceParameters.SequenceParameters(i)

            self.assertGreaterEqual(1, a.get_kappa())

    def test_phaseDiagramDefinitions(self):
        base = "EKEKEKEKEKEKEK"
        sequences = []
        phaseplotreg = []
        for extender in ['E', 'K', 'G', 'GK', 'GE']:
            for i in range(0, 50):
                if i > 20:
                    if i % 2 == 0:
                        continue
                if i > 30:
                    if i % 3 == 0:
                        continue

                sequences.append(
                    sequenceParameters.SequenceParameters(
                        base + extender * i))
                phaseplotreg.append(
                    sequenceParameters.SequenceParameters(
                        base + extender * i).get_phasePlotRegion())

        plots.save_multiple_phasePlot2(
            sequences,
            'tmpfiles/check_phasePlot_regions_match',
            phaseplotreg)


    def test_get_linear_NCPR(self):
        testSeq = sequenceParameters.SequenceParameters('ASLPEALPSEPEASPEPASLEAPLSEPLASEASEEKEKEKEKEKEKEKEKEKEKALSPELASPELASEKEKASLEAPSELAPSELALSELAPSEAPSEAL')
        self.assertEqual((np.array([ 0. ,  0. , -0.2, -0.2, -0.2, -0.2, -0.2, -0.2, -0.2, -0.4, -0.4,
                                    -0.4, -0.2, -0.4, -0.2, -0.2, -0.2, -0.2, -0.2, -0.2, -0.2, -0.2,
                                    -0.2, -0.2, -0.2, -0.2, -0.2, -0.2, -0.2, -0.2, -0.2, -0.4, -0.6,
                                    -0.2, -0.4, -0.2, -0.2,  0.2, -0.2,  0.2, -0.2,  0.2, -0.2,  0.2,
                                    -0.2,  0.2, -0.2,  0.2, -0.2,  0.2, -0.2,  0.2,  0. ,  0.2,  0. ,
                                    0.2, -0.2, -0.2, -0.2, -0.2, -0.2, -0.2, -0.2, -0.2, -0.2, -0.4,
                                    0. , -0.2,  0. ,  0. ,  0.2,  0. ,  0. , -0.2, -0.2, -0.2, -0.4,
                                    -0.2, -0.2, -0.2, -0.2, -0.2, -0.2, -0.2, -0.2, -0.2, -0.2, -0.2,
                                    -0.2, -0.2, -0.2, -0.2, -0.2, -0.2, -0.2, -0.4, -0.2, -0.2,  0. ,
                                    0. ]) == testSeq.get_linear_NCPR()[1]).all(), True)

    def test_get_linear_FCR(self):
        testSeq = sequenceParameters.SequenceParameters('ASLPEALPSEPEASPEPASLEAPLSEPLASEASEEKEKEKEKEKEKEKEKEKEKALSPELASPELASEKEKASLEAPSELAPSELALSELAPSEAPSEAL')
        self.assertEqual((np.array([ 0. ,  0. ,  0.2,  0.2,  0.2,  0.2,  0.2,  0.2,  0.2,  0.4,  0.4,
                                     0.4,  0.2,  0.4,  0.2,  0.2,  0.2,  0.2,  0.2,  0.2,  0.2,  0.2,
                                     0.2,  0.2,  0.2,  0.2,  0.2,  0.2,  0.2,  0.2,  0.2,  0.4,  0.6,
                                     0.6,  0.8,  1. ,  1. ,  1. ,  1. ,  1. ,  1. ,  1. ,  1. ,  1. ,
                                     1. ,  1. ,  1. ,  1. ,  1. ,  1. ,  1. ,  1. ,  0.8,  0.6,  0.4,
                                     0.2,  0.2,  0.2,  0.2,  0.2,  0.2,  0.2,  0.2,  0.2,  0.2,  0.4,
                                     0.4,  0.6,  0.8,  0.8,  0.6,  0.4,  0.4,  0.2,  0.2,  0.2,  0.4,
                                     0.2,  0.2,  0.2,  0.2,  0.2,  0.2,  0.2,  0.2,  0.2,  0.2,  0.2,
                                     0.2,  0.2,  0.2,  0.2,  0.2,  0.2,  0.2,  0.4,  0.2,  0.2,  0. ,
                                     0. ]) == testSeq.get_linear_FCR()[1]).all(), True)

    def test_get_linear_hydropathy(self):
        testSeq = sequenceParameters.SequenceParameters('ASLPEALPSEPEASPEPASLEAPLSEPLASEASEEKEKEKEKEKEKEKEKEKEKALSPELASPELASEKEKASLEAPSELAPSELALSELAPSEAPSEAL')
        self.assertEqual((np.array([ 0.        ,  0.        ,  0.49333333,  0.49333333,  0.59555556,
                                  0.47555556,  0.49333333,  0.49333333,  0.41777778,  0.25555556,
                                  0.33111111,  0.33111111,  0.37333333,  0.33111111,  0.37333333,
                                  0.37333333,  0.37333333,  0.49333333,  0.49333333,  0.56888889,
                                  0.49333333,  0.59555556,  0.49333333,  0.49333333,  0.41777778,
                                  0.53777778,  0.49333333,  0.49333333,  0.49333333,  0.56888889,
                                  0.46666667,  0.34888889,  0.28888889,  0.28      ,  0.16222222,
                                  0.09333333,  0.09333333,  0.08444444,  0.09333333,  0.08444444,
                                  0.09333333,  0.08444444,  0.09333333,  0.08444444,  0.09333333,
                                  0.08444444,  0.09333333,  0.08444444,  0.09333333,  0.08444444,
                                  0.09333333,  0.08444444,  0.21111111,  0.37333333,  0.44222222,
                                  0.48444444,  0.49333333,  0.53777778,  0.49333333,  0.49333333,
                                  0.49333333,  0.49333333,  0.49333333,  0.49333333,  0.49333333,
                                  0.45111111,  0.44222222,  0.28      ,  0.15333333,  0.21111111,
                                  0.27111111,  0.44222222,  0.44222222,  0.56888889,  0.49333333,
                                  0.49333333,  0.33111111,  0.49333333,  0.49333333,  0.49333333,
                                  0.49333333,  0.49333333,  0.49333333,  0.49333333,  0.61333333,
                                  0.61333333,  0.61333333,  0.61333333,  0.61333333,  0.49333333,
                                  0.49333333,  0.49333333,  0.44888889,  0.37333333,  0.39111111,
                                  0.33111111,  0.44888889,  0.49333333,  0.        ,  0.        ]) - testSeq.get_linear_hydropathy()[1] < 0.0001).all(), True)


    

                         



"""
   !--------------------------------------------------------------------------!
   ! LICENSE INFO:                                                            !
   !--------------------------------------------------------------------------!
   !    This file is part of localCIDER.                                      !
   !                                                                          !
   !    Version 0.1.9                                                         !
   !                                                                          !
   !    Copyright (C) 2014 - 2016                                             !
   !    The localCIDER development team (current and former contributors)     !
   !    Alex Holehouse, James Ahad, Rahul K. Das.                             !
   !                                                                          !
   !    localCIDER was developed in the lab of Rohit Pappu at Washington      !
   !    University in St. Louis. Please see the website for citation          !
   !    information:                                                          !
   !                                                                          !
   !    http://pappulab.github.io/localCIDER/                                 !
   !                                                                          !
   !    For more information please see the Pappu lab website:                !
   !                                                                          !
   !    http://pappulab.wustl.edu/                                            !
   !                                                                          !
   !    localCIDER is free software: you can redistribute it and/or modify    !
   !    it under the terms of the GNU General Public License as published by  !
   !    the Free Software Foundation, either version 3 of the License, or     !
   !    (at your option) any later version.                                   !
   !                                                                          !
   !    localCIDER is distributed in the hope that it will be useful,         !
   !    but WITHOUT ANY WARRANTY; without even the implied warranty of        !
   !    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         !
   !    GNU General Public License for more details.                          !
   !                                                                          !
   !    You should have received a copy of the GNU General Public License     !
   !    along with localCIDER.  If not, see <http://www.gnu.org/licenses/>.   !
   !--------------------------------------------------------------------------!
   ! AUTHORSHIP INFO:                                                         !
   !--------------------------------------------------------------------------!
   !                                                                          !
   ! MAIN AUTHOR:   Alex Holehouse                                            !
   !                                                                          !
   !--------------------------------------------------------------------------!



   File Description:
   ================

   backendtools contains various utilities and tools used by localCIDER. Especially
   useful is the fact all STDOUT is done through the various *_message methods
   defined here. This allows easy control over the degree of verbosity localCIDER
   displays. Verbocity is hard-coded into the config file



"""
import sys
import os
from config import HUSH_WARNINGS, HUSH_STATUS, HUSH_ALL

#...................................................................................#


def warning_message(message):
    if not HUSH_WARNINGS and not HUSH_ALL:
        print "WARNING: " + message


#...................................................................................#
def status_message(message):
    """
    Unless the HUSH_STATUS variable is set to True
    this function prints $message to STDOUT
    """
    if not HUSH_STATUS and not HUSH_ALL:
        print message


#...................................................................................#
def running_dotdotdot():
    """
    Print a single dot (period) to STDOUT without a newline.
    Useful for indicating progress.
    """
    if not HUSH_STATUS and not HUSH_ALL:
        sys.stdout.write('.')
        sys.stdout.flush()


#...................................................................................#
def warn_thisWillBeRemoved():
    """
    Prints a standard depreciation warning
    """
    warning_message("<> TO BE DEPRECATED IN THE NEAR FUTURE <>")


#...................................................................................#
def warn_notReadyYet():
    """
    Prints a standard 'this feature isn't ready yet' warning
    """
    warning_message("<> THIS FEATURE IS COMING IN THE NEXT VERSION <>")


#...................................................................................#
def return_absolute_datafile_path(filename):
    """
    Function which returns the absolute path
    of a file in the package's data directory.

    """
    warn_thisWillBeRemoved()

    # get the absolute path of where we are now
    absolute_path = os.path.realpath('__file__')
    splitted = os.path.split(absolute_path)

    print absolute_path

    # cut off the first two path
    for i in range(0, 2):
        splitted = os.path.split(splitted[0])

    return(os.path.join(splitted[0], "data", filename))


#...................................................................................#
def verifyType(obj, typeHere):
    """
    Function which takes an object and some type and ensures the
    object

    """

    try:
        objclass = obj.__class__
        if objclass == typeHere:
            return True
        else:
            return False
    except SyntaxError as e:
        # a primitive upon which .__class__ is invalid
        if isinstance(obj, typeHere):
            return True
        else:
            return False
